//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "GameScenario.h"
#include <iostream>
#include <fstream>
#include "MapCreationDialog.h"
#include "armyset.h"
#include "playerlist.h"
#include "citylist.h"
#include "city.h"
#include "ruin.h"
#include "armysetlist.h"
#include "stacklist.h"
#include "mapdef.h"
#include "player.h"

//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)


GameScenario::GameScenario(GameType gameType, MapType mapType)
	 :d_gameType(gameType), d_mapType(mapType), d_turns(0)
{
	mapdef = new Mapdef();

	// now let's load the correct mapset
	if ((d_mapType == RANDOM) || (d_mapType == PREVIOUS))
	       mapdef->load_mapset("../pic/map/default/default");	// random map chosen
	else mapdef->load_mapset(mapdef->extract_mapset(d_mapFile));


	MapCreationDialog mapCreationDialog(d_mapType, d_mapFile, 0,
        PG_Rect((PG_Application::GetScreenWidth() - 250)/2,
                (PG_Application::GetScreenHeight() - 105)/2, 250, 105));
	mapCreationDialog.Show();
	mapCreationDialog.RunModal();
	mapCreationDialog.Hide();

	initGameType();
}

GameScenario::~GameScenario()
{
	delete mapdef;
}

void GameScenario::initGameType()
{
	switch (d_gameType)
	{
		case SINGLE_PLAYER:
		{
			initSinglePlayer();
			break;
		}
		case SCENARIO:
        {	
			initScenario();
			break;
		}
		case MULTI_PLAYER:
		{
			initMultiPlayer();
			break;
		}
	}
}

bool GameScenario::isMultiPlayer() const
{
	return d_gameType == MULTI_PLAYER;
}

void GameScenario::initSinglePlayer()
{
	debug("initSinglePlayer()");

	checkInitialisation();
	loadArmysets();
	assignPlayers();
	produceFirstArmy();
	misc();

	//quick hack: add a hero to each player
	//we know that some cities already belong to the players so we use
	//them to set up our hero
	for (Citylist::iterator c_it = citylist->begin(); c_it != citylist->end();
				c_it++)
	{
		if (c_it->get_player()->get_name() != "Neutral")
			c_it->add_hero();
			//Ok, all heroes are now called Demonlord, but that will
			//do it for now
	}
}

void GameScenario::initScenario()
{
	//scenarios differ from simple games by the fact that players, player-owned
	//cities etc. are already given at game start and (later) by different
	//objectives, and are also part of campaigns. Currently, you would
	//hardly benefit from scenarios, so they aren't implemented yet :).
}

void GameScenario::initMultiPlayer()
{
	/*P
	client->wait_for_map();
	mapdef->random();

	client->wait_for_playerorder();
	load_armyset();

	if (client->is_first())
	{
		assign_players();
		produce_first_army();
	}

	//we just assume, the map from the server is already smoothed, so we
	//don't need to do so
	*/
}

void GameScenario::loadArmysets()
{
	debug("loadArmysets()");

	// default armyset HAS to be loaded because the neutral player uses it
    new Armyset("default");
	
	for (Playerlist::iterator it = playerlist->begin(); it != playerlist->end(); it++)
	{
		// check if the armyset exists
		if (!armysetlist->load_armyset((*it).get_armyset_name()))
		{
			std::cerr << "GameScenario: couldn't load armyset!" << endl;
            std::cerr << "Exiting FreeLords" << endl;
			exit(-1);
		}

		// if the armyset of this player hasn't been loaded before load it
	}
}

// Create a stack in each city with the weakest army the city can produce

void GameScenario::produceFirstArmy()
{
	debug("produceFirstArmy()");
	for (Citylist::iterator it = citylist->begin(); it != citylist->end(); it++)
	{
		(*it).set_production(0);
		(*it).produce_army();
	}
}

// assign each player a capital

void GameScenario::assignPlayers()
{
	debug("assignPlayers()");
	Citylist::iterator c_it = citylist->begin();
	for (Playerlist::iterator it = playerlist->begin(); it != playerlist->end(); it++)
	{
		(*c_it).set_player(&(*it));
		c_it++;
	}
}

void GameScenario::checkInitialisation()
{
	if (citylist->size() == 0)
	{
		std::cerr << "ERROR: There is no city on this map! Please use the random map feature\n";
		exit(-1);
	}	       
}

void GameScenario::nextTurn()
{
	if (++d_turns != 1)
	{
		citylist->nextTurn();
		stacklist->nextTurn();
	}
}

//a function as quick hack; we need to do some rework anyway,so don't bother
void GameScenario::misc()
{
	debug("GameScenario::misc")
	//create a strong stack

	for (Ruinlist::iterator it = ruinlist->begin(); it != ruinlist->end(); it++)
	{
		Stack* keeper = new Stack(0, (*it).get_pos());
		keeper->push_back(new Army((*armysetlist->begin()).second, 9));
		(*it).set_occupant(keeper);
	}
}
