//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GAME_SCENARIO_H
#define GAME_SCENARIO_H

#include <paragui.h>
#include <pgeventobject.h>
#include <pgmessageobject.h>
#include <string>

class GameScenario : public PG_EventObject, public PG_MessageObject
{
	public:
		enum GameType {SCENARIO, SINGLE_PLAYER, MULTI_PLAYER};
		enum MapType {RANDOM, LOADED, PREVIOUS};

		// CREATORS
		GameScenario(GameType gameType, MapType mapType);
		~GameScenario();

		// MANIPULATORS
		void setMapFile(std::string mapFile);
		void setMame(std::string name);
		void checkInitialisation();
		void nextTurn();
		void initGameType();
	
		// ACCESSORS
		int getTurns() const {return d_turns;}
		MapType getMapType() const {return d_mapType;}
		GameType getGameType() const {return d_gameType;}
		std::string getName() const {return d_name;}
		bool isMultiPlayer() const;
		
	private:
		// MANIPULATORS
		void loadArmysets();
		void produceFirstArmy();
		void assignPlayers();
		void misc();
		void initSinglePlayer();
		void initScenario();
		void initMultiPlayer();
		void cleanup();
		
		// DATA
		GameType d_gameType;
		MapType d_mapType;
		std::string d_mapFile;
		int d_turns;
		std::string d_name;
};

#endif // GAME_SCENARIO_H

