//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "army.h"
#include "armyset.h"

Army::Army(Armyset* armyset, int type)
	:Armyset_army(armyset->army[type]), d_armyset(armyset), d_type(type),
	d_hp(get_max_hp()), d_moves(get_max_moves()), d_strength(get_max_strength()), 
	d_xp(0), d_level(1), d_blessed(false), d_grouped(true)
{
	static unsigned int counter = 0;
	d_id = (++counter);
}

Army::Army(Army& a)
	:Armyset_army(a), d_armyset(a.d_armyset), d_id(a.d_id),
	d_type(a.d_type), d_hp(a.d_hp), d_moves(a.d_moves), 
	d_strength(a.d_strength), d_xp(a.d_xp), d_level(a.d_level),
	d_blessed(a.d_blessed), d_grouped(a.d_grouped)
{
}

Army::~Army()
{
}

void Army::reset_moves()
{
	d_moves = get_max_moves();
}

void Army::bless()
{
	if (!d_blessed)
	{
		d_blessed = true;
		d_strength++;
	}
}

void Army::heal(int hp)
{
    if (hp > 0)
    {
	//add no_hp
	d_hp += hp;
    }
    else
    {
	//healing at end of turn, army has to take care of how much to heal
	//for the beginning, we heal 1/5th of hp's every turn
	// (should be changed somewhen)
	d_hp += (get_max_hp() / 5);
    }

    if (d_hp > get_max_hp())
	d_hp = get_max_hp();
}

bool Army::damage (int damage_done)
{
	d_hp -= damage_done;
	if (d_hp <= 0) return true;
	else return false;
}

void Army::decrement_moves(int moves)
{
	d_moves -= moves;

	//just to make sure
	if (d_moves < 0)
	{
		cerr <<"Error: moves of army < 0\n";
		d_moves = 0;
	}
}

void Army::gain_xp(int n)
{
	//How do we want to handle experience points?
	d_xp += n;
}

