//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef ARMY_H
#define ARMY_H

#include "armyset_army.h"

class Armyset;

// This class contains all data and operations related to an Army. It's mostly
// used by the class Stack

class Army : public Armyset_army
{
	public:
		// CREATORS
		Army(Armyset* armyset, int type);
		Army(Army& a);
		~Army();

		// MANIPULATORS
		void set_type(int type){d_type = type;}
		void set_hp(int hp) {d_hp = hp;}
		void set_grouped(bool grouped){d_grouped = grouped;}
 		void heal(int hp = 0);
		bool damage(int damage_done);		//true, if army has died
		void decrement_moves(int moves);
		void reset_moves();
		void bless();
		void gain_xp(int n);

		// ACCESSORS
		int get_id() const {return d_id;}
		int get_type() const {return d_type;}
		int get_hp() const {return d_hp;}
		int get_moves() const {return d_moves;}
		int get_strength() const {return d_strength;}
		int get_xp() const {return d_xp;}
		int get_level() const {return d_level;}
		bool is_blessed() const {return d_blessed;}
		bool is_grouped() const {return d_grouped;}

	private:
		Armyset* d_armyset;
		unsigned int d_id;
		int d_type;
		int d_hp;
		int d_moves;
		int d_strength;
		int d_xp;			// experience points
		int d_level;
		bool d_blessed;			// if army is blessed then strength is +1
		bool d_grouped;
};

#endif // ARMY_H
