//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef ARMYSET_ARMY_H
#define ARMYSET_ARMY_H

#include <SDL.h>
#include <string>

// This class is the atom of every armyset. It contains all data related to
// a single armytype of an armyset

class Armyset_army
{
	public:
		// CREATORS
                Armyset_army();
		Armyset_army(const Armyset_army& a);
		~Armyset_army();

		// MANIPULATORS
		void set_name(std::string name){d_name = name;}
		void set_max_hp(int max_hp){d_max_hp = max_hp;}
		void set_max_strength(int max_strength){d_max_strength = max_strength;}
		void set_defense(int defense){d_defense = defense;}
		void set_vitality(int vitality){d_vitality = vitality;}
		void set_max_moves(int max_moves){d_max_moves = max_moves;}
		void set_upkeep(int upkeep){d_upkeep = upkeep;}
		void set_production(int production){d_production = production;}
		void set_production_cost(int production_cost){d_production_cost = production_cost;}
		void set_move_bonus(unsigned int move_bonus){d_move_bonus = move_bonus;}
		void set_army_bonus(unsigned int army_bonus){d_army_bonus = army_bonus;}
		void set_pixmap(SDL_Surface* pixmap){d_pixmap = pixmap;}
		void set_xp_worth (int xp_worth){d_xp_worth = xp_worth;}
		void print_debug_info();
		
		// ACCESSORS
		SDL_Surface* get_pixmap() const {return d_pixmap;}
		std::string get_name() const {return d_name;}
		int get_max_strength() const {return d_max_strength;}
		int get_max_moves() const {return d_max_moves;}
		int get_upkeep() const {return d_upkeep;}
		int get_production() const {return d_production;}
		int get_defense() const {return d_defense;}
		int get_vitality() const {return d_vitality;}
		int get_production_cost() const {return d_production_cost;}
		int get_max_hp() const {return d_max_hp;}
		unsigned int get_move_bonus() const {return d_move_bonus;}
		unsigned int get_army_bonus() const {return d_army_bonus;}
		int get_xp_worth() const {return d_xp_worth;}
		
	private:
		std::string d_name;
		SDL_Surface* d_pixmap;
		int d_production;		// needed turns to produce this armytype
		int d_production_cost;
		int d_upkeep;
		int d_max_strength;
		int d_defense;
		int d_max_hp;
		int d_max_moves;
		int d_vitality;
		unsigned int d_move_bonus;
		unsigned int d_army_bonus;
		int d_xp_worth;
};

#endif // ARMYSET_ARMY_H

