//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <iostream>
#include <fcntl.h>
#include <unistd.h>
#include <expat.h>
#include <SDL_image.h>
#include "armysetlist.h"
#include "armyset.h"
#include "armyset_army.h"
#include "file.h"

#define BUFF_SIZE 1000

//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

Armysetlist* armysetlist;

Armysetlist::Armysetlist()
	: std::map<std::string, Armyset*>()
{
}

Armysetlist::~Armysetlist()
{
}

void expat_callback(void *data, const char *el, const char **attr)
{
	debug("expat_callback()");

	Armyset* armyset = (Armyset*)(data);
	std::string sel(el);

	if (sel == "Army")
	{
		int k = 0;

		Armyset_army army;

		while (attr[k])
		{
			if (std::string(attr[k]) == "Name")
				army.set_name(std::string(attr[k+1]));
			else if (std::string(attr[k]) == "Production")
				army.set_production(atoi(attr[k+1]));
			else if (std::string(attr[k]) == "Production_cost")
				army.set_production_cost(atoi(attr[k+1]));
			else if (std::string(attr[k]) == "Upkeep")
				army.set_upkeep(atoi(attr[k+1]));
			else if (std::string(attr[k]) == "Strength")
				army.set_max_strength(atoi(attr[k+1]));
			else if (std::string(attr[k]) == "Defense")
				army.set_defense(atoi(attr[k+1]));
			else if (std::string(attr[k]) == "Hitpoints")
				army.set_max_hp(atoi(attr[k+1]));
			else if (std::string(attr[k]) == "Moves")
				army.set_max_moves(atoi(attr[k+1]));
			else if (std::string(attr[k]) == "Vitality")
				army.set_vitality(atoi(attr[k+1]));
			else if (std::string(attr[k]) == "Move_bonus")
				army.set_move_bonus((unsigned int)(atoi(attr[k+1])));
			else if (std::string(attr[k]) == "Army_bonus")
				army.set_army_bonus((unsigned int)(atoi(attr[k+1])));
			else if (std::string(attr[k]) == "Picture")
			{
				std::string picfile = file::get_army_pic(armyset->get_name(), std::string(attr[k+1]));
				army.set_pixmap(IMG_Load(picfile.c_str()));
			}
			k = k + 2;	// always one attribute with one value
		}

		armyset->army.push_back(army);
	}
}

Armyset* Armysetlist::get_armyset(std::string name)
{
	// this one is rather easy thanks to the STL
	// if the name entry doesn't exist, std::map creates one and initialises with 0

	return (*this)[name];
}

bool Armysetlist::load_armyset(std::string name)
{
	debug("load_armyset()\n");

	//first, look if Armyset already exists
	if ((*this)[name] != 0)
		return true;

	//no, so create a new Armyset first
	Armyset *myset = new Armyset(name);

	//now set up the parser and the armyfile input stream...
	XML_Parser myparser = XML_ParserCreate(0);

	if (!myparser)
	{
		std::cerr <<"Armysetlist: Couldn't allocate memory for XML parser\n";
		exit (1);
	}

	XML_SetStartElementHandler(myparser, expat_callback);
	XML_SetUserData(myparser, (void*) myset);

	int docfd = open(file::get_army_file(name).c_str(), O_RDONLY);

	for (;;)
	{
		void* buff = XML_GetBuffer(myparser, BUFF_SIZE);

		if (buff == NULL) cerr << "Can't allocate memory\n";

		int bytes_read = read(docfd, buff, BUFF_SIZE);

		if (bytes_read == 0) break;

		if (XML_ParseBuffer(myparser, bytes_read, bytes_read == 0))
		{
			debug("Error parsing Armyset " << name << " on line "
				<< XML_GetCurrentLineNumber(myparser) << ": "
				<< XML_ErrorString(XML_GetErrorCode(myparser)));
				
		}
	}

	close(docfd);
	XML_ParserFree(myparser);

	(*this)[name] = myset;
	return true;
}

bool Armysetlist::erase_by_name(std::string name)
{
	iterator it = find(name);
	//armyset exists
	if (it != end())
	{
	//	delete (*this)[name];
		erase(it);
		return true;
	}
	return false;
}

