//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "city.h"
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "army.h"
#include "stacklist.h"
#include "player.h"
#include "armyset.h"
#include "defs.h"

//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

City::City(Player* player, PG_Point pos, std::string name, int gold)
	:Location(name, pos, 2), d_player(player), d_duration(-1),
	d_production(-1), d_gold(gold), d_burnt(false), d_defense_level(1)
{
	//assign id
	static unsigned int counter = 0;
	d_id = (++counter);

	// Initialise armytypes
	for (int i = 0; i < 4; i++) d_armytype[i] = -1; 
}

City::~City()
{
}

void City::next_turn()
{
	// check if an army should be produced
	if (d_production >= 0 && --d_duration == 0) 
	{
		produce_army();
	}

	// add goldproduction of city to the gold of owner
	d_player->addGold(d_gold);	
}

// Find a stack that isn't full on one of the 4 possible citytiles
// or create a new one if necessary. If there isn't enough room
// for a new stack return 0

Stack* City::get_free_stack()
{
	for (int i = 0; i < 2; i++)
		for (int j = 0; j < 2; j++)
		{
			Stack* stack = stacklist->get_object_at(get_pos().x + j, get_pos().y+ i);

			// No stack found so create one
			if (stack == 0)
			{
				PG_Point temp;
				temp.x = get_pos().x + j;
				temp.y = get_pos().y + i;
				stack = new Stack(d_player, temp);
				stacklist->push_back(stack);
				return stack;
			}
			else if (stack->size() < 8) return stack;
		}

	return 0;
}

// add a hero to a free stack

void City::add_hero(int hero_gender, std::string hero_name)
{
	Stack* stack = get_free_stack();

	// No stack found so create one
	if (!stack)
	{
		stack = new Stack(d_player, get_pos());
		stacklist->push_back(stack);
	}

	// add hero to stack
	Army* hero = new Army(d_player->get_armyset(), hero_gender);
	hero->set_name(hero_name);
	stack->push_front(hero);
}

// produce a new army and add it to a free stack
// if there is no free stack set the same production
// again and hope that there is enough space later

void City::produce_army()
{
	debug(cdbg << "produce_army()\n");
	Stack* stack = get_free_stack();

	if (stack)
	{
		// add produced army to stack
		stack->push_back(new Army(d_player->get_armyset(), d_armytype[d_production]));
	}
	// start producing next army of same type
	set_production(d_production);
}

void City::set_production(int production)
{
	d_production = production;

	if (production >= 0)
	{
		// set the duration to produce this armytype
		d_duration = d_player->get_armyset()->army[d_armytype[d_production]].get_production(); 
	}
}


bool City::is_friend(Player* player) const
{
	if (d_player == player) return true;
	else return false;
}

// set the possible armyproductions of the city randomly

void City::set_random_armytypes()
{
	d_armytype[0] = 2;
	int random_nr = rand() % 2;
	if (random_nr == 1) 
	{
		d_armytype[1] = 3;
		random_nr = rand() % 4;
		if (random_nr == 3) d_armytype[2] = 4 + (rand() % 3);
	}
}

bool City::have_production(int type)
{
	for (int i = 0; i < 4; i++)
	{
		if (d_armytype[i] == type) return true;
	}
	return false;
}

