//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef CITY_H
#define CITY_H

#include <string>
#include "Location.h"

class Player;
class Stack;

class City : public Location
{
	public:
		// CREATORS
		City(Player* p, PG_Point pos, std::string name = 0, int gold = 30);
		~City();

		// MANIPULATORS
		void set_player(Player* player){d_player = player;}
		void set_gold(int gold){d_gold = gold;}
		void set_defense_level(int defense_level){d_defense_level = defense_level;}
		void set_production(int production);
		void set_duration(int duration){d_duration = duration;}
		void set_armytype(int index, int armytype){d_armytype[index] = armytype;}
		void set_burnt(bool burnt){d_burnt = burnt;}
		void set_random_armytypes();
		void produce_army();
		void add_hero(int hero_gender = 0, std::string hero_name = "Demonlord");
		void next_turn();
		bool have_production(int type);

		// ACCESSORS
		unsigned int get_id() {return d_id;}
		int get_defense_level() const {return d_defense_level;}
		Player* get_player() const {return d_player;}
		int get_duration() const {return d_duration;}
		int get_production() const {return d_production;}
		int get_gold() const {return d_gold;}
		int get_armytype(int slot) const {return d_armytype[slot];}
		bool is_burnt() const {return d_burnt;}
		bool is_friend(Player* player) const;

	private:
		// MANIPULATORS
		Stack* get_free_stack();

		// DATA
		Player* d_player;		// Owner
		int d_duration;			// needed turns to finish current production
		int d_armytype[4];		// possible productiontypes
		int d_production;		// number of produced armytype
		int d_gold;			// gold produced by city each turn
		bool d_burnt;			// is city burnt down?
		int d_defense_level;		// defense of the city
		unsigned int d_id;
};

#endif // CITY_H
