//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "client.h"
#include "turninfo.h"
#include "playerlist.h"

//#define DEBUG_IT

#ifdef DEBUG_IT
	#define debug(x) {x;}
#else
	#define debug(x)
#endif

#define cdbg cerr<<__FILE__<<": "<<__LINE__<<": "

Client* client;

Client::Client(QString host, Q_UINT16 port)
	:QSocket()
{
	connectToHost(host, port);
	first = false;
	already_connected = false;
}

Client::~Client()
{
}

void Client::send_playerinfo(QString name)
{
	// create the playerinfo
	QString data("PLAYER\n");
	data += name + "\n";
	
	if (writeBlock(data, data.length()) == -1) 
		debug(cdbg << "client couldn't write to socket\n");

	// now really send data to host
	flush();
}

// Wait until the server has sent the map completly 
void Client::wait_for_map()
{
	debug(cdbg << "wait_for_map()\n");

	QFile f("random.map");
	if (f.open(IO_WriteOnly))
	{
		//f.at(f.size());		// set file index to end of file for APPEND mode
		QTextStream t(&f);
		QString str;
		do
		{
			while (!canReadLine()) waitForMore(-1);
			str = readLine();
			if (str != "END\n") t << str;
		}
		while (str != "END\n");
		f.close();
	}
	else cerr << "couldn't write file random.map\n";
}

void Client::wait_for_playerorder()
{
	debug(cdbg << "wait_for_playerorder()\n");
	
	while(!canReadLine()) waitForMore(-1);
	QString str(readLine());

	if (str == "FIRST\n") first = true;
	if (str == "NOT_FIRST\n") first = false;

	playerlist.clear();
	while (str != "END\n")
	{
		while (!canReadLine()) waitForMore(-1);
		str = readLine();
		if (str != "END\n")
		{
			// all of the players name except the "\n"
			str.truncate(str.length()-1);
			playerlist.add_new_player(str, "default");
		}
	}
}

void Client::wait_for_turninfo()
{
	debug(cdbg << "wait_for_turninfo()\n");

	if (canReadLine())
	{
		QString str(readLine());
		if (str == "TURNINFO\n") read_turninfo();
	}

	if (!already_connected)
	{
		connect(this, SIGNAL(readyRead()), this, SLOT(incoming_data()));
		already_connected = true;
	}
}

void Client::read_turninfo()
{
	debug(cdbg << "read_turninfo()\n");

	QFile* f = turninfo->get_file();
	QTextStream t(f);
	if (f->open(IO_WriteOnly))
	{
		QString str;
		while (str != "END\n")
		{
			while (!canReadLine()) waitForMore(-1);
			str = readLine();
			if (str != "END\n") t << str;
		}
		f->close();
		emit got_turninfo();
	}
	else cerr << "couldn't open turninfo.dat\n";
}

void Client::incoming_data()
{
	debug(cdbg << "incoming_data()\n");
	
	while(!canReadLine()) waitForMore(-1);
	QString str(readLine());		// this identifies the incoming message
	if (str == "TURNINFO\n") read_turninfo();
}

void Client::send_turninfo()
{
	debug(cdbg << "send_turninfo()\n");

	QFile* f = turninfo->get_file();
	QTextStream t(f);
	if (f->open(IO_ReadOnly))
	{
		QString str;
		while (!t.eof()) str += t.readLine() + "\n";		// this excludes the "\n"
		client->writeBlock(str, str.length());
		client->flush();
		f->close();
		f->remove();
	}
	else cerr << "couldn't open turninfo.dat\n";
}
