//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


//This file contains the various macros used within freelords.

#ifndef DEFINITIONS_H
#define DEFINITIONS_H

#include <paragui.h>
#include <list>

//ARMY_* these macros define special abilities of different armies. 

#define ARMY_NONE 0
#define ARMY_SHIP (1<<1)	//unit can only move on water
#define ARMY_HERO (1<<2)	//units has hero bonus (addit. strength for 
				//other units in combat)
#define ARMY_CAVALRY (1<<3)	//cavalry unit (+1 strength on grass)
#define ARMY_ANTICAVALRY (1<<4)	//double strength against cavalry
#define ARMY_HEALFAST (1<<5)	//+1 HP every 2nd fight turn and at end of fight
#define ARMY_EVADE (1<<6)	//escape first attack during battle round
#define ARMY_CRITICAL (1<<7)	//unit can do critical hits (instant death of
				//enemy regardless of hitpoints)
#define ARMY_HAS_HERO (1<<8)	//there's a hero in the stack (used during 
				//battle calculations
#define ARMY_IS_LOADED (1<<9)	//army is transported by another unit
#define ARMY_TRANSPORT_1 (1<<10)//army can hold one other unit
#define ARMY_TRANSPORT_2 (1<<11)//the same with two...
#define ARMY_TRANSPORT_3 (1<<12)//...and three units

#define ARMY_TRANSPORT ARMY_TRANSPORT_1&ARMY_TRANSPORT_2&ARMY_TRANSPORT_3

//-----------------------------------------------------------------------------
//These macros qualify terrain and armies. If an army and the maptile it moves 
//on have the same move bonus set, the unit needs two move points on that terain
//regardless of the number of MP's the terrain typically demands

#define MAPTILE_NONE 0
#define MAPTILE_WATER 1
#define MAPTILE_FOREST 2
#define MAPTILE_HILL 4
#define MAPTILE_SWAMP 8


//-----------------------------------------------------------------------------
//some typedefs for quests

typedef struct
{
	unsigned int type;
	unsigned int subtype;
	int amount;
} fl_Treasure;

#define TREASURE_GOLD 1

//-----------------------------------------------------------------------------
//typedefs and macros for the actionlist (i.e. player's lists which hold 
//information about the player's actions)

#define ACTION_STACK_MOVED_ONE_STEP 1
#define ACTION_STACK_SPLIT 2
#define ACTION_STACK_SEARCH_RUIN 3
#define ACTION_STACK_FIGHT 4
#define ACTION_STACK_FIGHT_CITY 5
#define ACTION_STACK_JOIN 6
#define ACTION_STACK_VISIT_TEMPLE 7
#define ACTION_CITY_OCCUPY 8
#define ACTION_CITY_RAZE 9
#define ACTION_CITY_UPGRADE_DEFENSE 10
#define ACTION_CITY_BUY_PRODUCTION 11
#define ACTION_CITY_CHANGE_PRODUCTION 12
#define ACTION_REWARD 13

typedef struct
{
	unsigned int action;
	std::list<int> data;
} data_package;

//-----------------------------------------------------------------------------
// The following macros are used to identify the messages coming from the
// various PG_MessageObjects! We can only send a message so the data has
// to decide which events it is

/* First of all, forget the above stuff. The real, new message handling mode 
 * beginning with paragui 0.7.1 comes here. If you define additional messages
 * please use the following conventions:
 * - name of the message: MSG_<class name>_<message name>
 * 	(we won't use the messages too often, so don't mind if the name is rather
 * 	epic) :)
 * - for each class, define only one MSG_USER_X (with X being a number from 
 * 	1 to 10), else you get an ambiguity!!!
 *
 * If you add new messages and callback functions, don't forget to derive the 
 * classes from PGMessageObject (for sending messages) and PG_EventObject 
 * (for defining callback functions).
 */

//for Qt compatibility (besides, it is shorter) :)
#define emit(a_message_type) SendMessage (this, a_message_type, 0, 0)

#define MSG_SMALLMAP_MOVED_VIEWRECT MSG_USER_1

#define MSG_BIGMAP_MOVED_VIEWRECT MSG_USER_1
#define MSG_BIGMAP_STACK_SELECTED MSG_USER_2

#define MSG_STACKLIST_STACK_MOVED MSG_USER_1
#define MSG_STACKLIST_STACK_DIED  MSG_USER_2
#define MSG_STACKLIST_CITY_OCCUPIED MSG_USER_3
#define MSG_STACKLIST_REDRAW MSG_USER_4

#define MSG_PLAYERLIST_GAME_WON MSG_USER_1
#define MSG_PLAYERLIST_PLAYER_DEAD MSG_USER_2
#define MSG_PLAYERLIST_NEXT_TURN MSG_USER_3

#define MSG_PLAYER_STATUS_CHANGED MSG_USER_1
#define MSG_PLAYER_LOCK_SCREEN MSG_USER_2
#define MSG_PLAYER_UNLOCK_SCREEN MSG_USER_3
#define MSG_PLAYER_STACK_UPDATE MSG_USER_5
#define MSG_PLAYER_CITY_UPDATE MSG_USER_6
#define MSG_PLAYER_STACK_DIED MSG_USER_7

#define MSG_MAINWINDOW_QUIT MSG_USER_1

#endif // DEFINITIONS_H
