//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FIGHT_H
#define FIGHT_H

#include <vector>
#include <deque>
#include <paragui.h>
#include "stack.h"

class Army;

class Fight
{ 
        public:
		enum RESULT {NONE = 0, ATTACKER_WON, DEFENDER_WON};
		
                // CREATORS
		Fight(Stack* attacker, Stack* defender, bool def_bonus = true);
		Fight(std::vector<Stack*>& attacker,
			std::vector<Stack*>& defender, bool def_bonus = true);
		~Fight();

                // MANIPULATORS
		bool new_round();	//return false on timeout
		RESULT next_fight();	//do one fight

		RESULT next_round();	//do a complete round
		
		// ACCESSORS
		bool is_round_finished() const {return d_round_finished;}

		// CONSTANTS
		static const int ATTACKER_MAX = 1;
		static const int DEFENDER_MAX = 4;
		static const int MAX_ROUNDS = 30;
		static const int MOVES_FOR_FIGHT = 3;

        private:
		bool fight_armies(Army* culprit, Army* victim, bool attack);
		void calculate_defense_bonus(Stack* stack);
		void check_hero(std::deque<Army*>& fighters);
		bool has_hit(float att_strength, float def_strength);

		// DATA
		std::deque<Army*> d_attacker;
		std::deque<Army*> d_defender;

		PG_Point d_pos;
		int d_turn;
		float d_add_defense;		//defense bonus of defender
		bool d_open_fight;		//is it a fight on open terrain

		bool d_round_finished;
		bool d_attacker_finished;
		std::deque<Army*>::iterator d_active;
		RESULT d_result;
};

#endif // FIGHT_H

