//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <assert.h>
#include <SDL_image.h>
#include <iostream>
#include <fstream>
#include "mapdef.h"
#include "mapset_tile.h"
#include "citylist.h"
#include "city.h"
#include "ruin.h"
#include "temple.h"
#include "playerlist.h"

//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

Mapdef* mapdef;

Mapdef::Mapdef()
{
}

Mapdef::~Mapdef()
{
	for (int i = 0; i < 100; i++)
	{
		for (int j = 0; j < 100; j++)
		{
            delete map[i][j];
		}
	}
}

void Mapdef::random()
{
	debug("random()\n");
	
	// Get city database ------------------------------------------
	//

	int name_counter;
	std::ifstream citynames("../dat/citynames"); // open citydatabase
	citynames >> name_counter;
	
	char city_database[name_counter][40];
	
	for (int i = 0; i < name_counter; i++)
		citynames.getline(city_database[i], 50);

	//
	// ------------------------------------------------------------
	
	char c;					// Temp variable
	int type;

	std::ifstream map_file("random.map");	// Open file

	for (int j = 0; j < 100; j++)
		for (int i = 0; i < 100; i++)
		{
			map_file >> c;

			switch (c)
			{
				case '~':
				{
					type = 1;
					break;
				}
				case '$':
				{
					type = 2; 
					break;
				}
				case 'h':
				{
					type = 3;
					break;
				}
				case 'M':
				{
					type = 4;
					break;
				}
				case '_':
				{
					type = 5;
					break;
				}
				default:
				{
					type = 0;
					break;
				}
			}

			//debug("creating maptile[" << i << "][" << j << "]: " << c);

			map[i][j] = new Maptile(i, j, type);

			//map[i][j]->print_debug_info();
		}
	
	for (int j = 0; j < 99; j++)
	{	
		// get the final newline
		map_file.get(c);

		for (int i = 0; i < 99; i++)
		{
			map_file.get(c);

			PG_Point p;
			p.x = i;
			p.y = j;

			switch (c)
			{
				case 'C':
				{
					int random_name_nr = rand() % name_counter;
					City city(playerlist->get_neutral(), p, city_database[random_name_nr]);
					city.set_random_armytypes();
					citylist->push_back(city);

					// change settings for the maptiles the city is on
					for (int a = 0; a < 2; ++a)
						for (int b = 0; b < 2; ++b)
						{
							map[p.x+a][p.y+b]->set_moves(1);
							map[p.x+a][p.y+b]->set_building(Maptile::CITY);
						}
					break;
				}
				case 'r':
				{
					ruinlist->push_back(Ruin(p));
					map[p.x][p.y]->set_building(Maptile::RUIN);
					break;
				}
				case 'T':
				{

					templelist->push_back(Temple(p));
					map[p.x][p.y]->set_building(Maptile::TEMPLE);
					break;
				}
			}
		}
	}
}

// Calculate the correct transition for the given maptile

void Mapdef::smooth(int x, int y)
{
	debug("smooth()\n");

	bool l, t, r, b, lt, rt, rb, lb;

	int type = map[x][y]->get_type();
	int tile = 5;

	if (map[x-1][y]->get_type() != type) l = true;else l = false;
	if (map[x][y-1]->get_type() != type) t = true;else t = false;
	if (map[x+1][y]->get_type() != type) r = true;else r = false;
	if (map[x][y+1]->get_type() != type) b = true;else b = false;
	if (map[x-1][y-1]->get_type() != type) lt = true;else lt = false;
	if (map[x+1][y-1]->get_type() != type) rt = true;else rt = false;
	if (map[x+1][y+1]->get_type() != type) rb = true;else rb = false;
	if (map[x-1][y+1]->get_type() != type) lb = true;else lb = false;

	if ((l) && (r) && (t) && (b)) tile = 0;
	if ((l) && (!r) && (t) && (!b)) tile = 1;
	if ((!l) && (!r) && (t) && (!b)) tile = 2;
	if ((!l) && (r) && (t) && (!b)) tile = 3;
	if ((l) && (!r) && (!t) && (!b)) tile = 4;
	if ((!l) && (r) && (!t) && (!b)) tile = 6;
	if ((l) && (!r) && (!t) && (b)) tile = 7;
	if ((!l) && (!r) && (!t) && (b)) tile = 8;
	if ((!l) && (r) && (!t) && (b)) tile = 9;
	if ((!l) && (!r) && (!t) && (!b) && (lt) && (!rt) && (!rb) && (!lb)) tile = 10;
	if ((!l) && (!r) && (!t) && (!b) && (!lt) && (rt) && (!rb) && (!lb)) tile = 11;
	if ((!l) && (!r) && (!t) && (!b) && (!lt) && (!rt) && (!rb) && (lb)) tile = 12;
	if ((!l) && (!r) && (!t) && (!b) && (!lt) && (!rt) && (rb) && (!lb)) tile = 13;
	if ((!l) && (!r) && (!t) && (!b) && (lt) && (rt) && (!rb) && (!lb)) tile = 14;
	if ((!l) && (!r) && (!t) && (!b) && (!lt) && (rt) && (rb) && (!lb)) tile = 15;
	if ((!l) && (!r) && (!t) && (!b) && (!lt) && (!rt) && (rb) && (lb)) tile = 16;
	if ((!l) && (!r) && (!t) && (!b) && (lt) && (!rt) && (!rb) && (lb)) tile = 17;
	if ((l) && (r) && (!t) && (!b)) tile = 18;
	if ((l) && (r) && (t) && (!b)) tile = 19;
	if ((l) && (r) && (!t) && (b)) tile = 20;
	if ((!l) && (!r) && (t) && (b)) tile = 21;
	if ((l) && (!r) && (t) && (b)) tile = 22;
	if ((!l) && (r) && (t) && (b)) tile = 23;
	if ((!l) && (!r) && (!t) && (!b) && (lt) && (rt) && (rb) && (lb)) tile = 24;
	if ((!l) && (!r) && (t) && (!b) && (rb) && (lb)) tile = 25;
	if ((!l) && (r) && (!t) && (!b) && (lt) && (lb)) tile = 26;
	if ((!l) && (!r) && (!t) && (b) && (lt) && (rt)) tile = 27;
	if ((l) && (!r) && (!t) && (!b) && (rt) && (rb)) tile = 28;
	if ((!l) && (!r) && (t) && (!b) && (rb) && (!lb)) tile = 29;
	if ((!l) && (!r) && (t) && (!b) && (!rb) && (lb)) tile = 30;
	if ((!l) && (!r) && (!t) && (b) && (!lt) && (rt)) tile = 31;
	if ((!l) && (!r) && (!t) && (b) && (lt) && (!rt)) tile = 32;
	if ((l) && (!r) && (!t) && (!b) && (!rt) && (rb)) tile = 33;
	if ((!l) && (r) && (!t) && (!b) && (!lt) && (lb)) tile = 34;
	if ((l) && (!r) && (!t) && (!b) && (rt) && (!rb)) tile = 35;
	if ((!l) && (r) && (!t) && (!b) && (lt) && (!lb)) tile = 36;
	if ((!l) && (!r) && (!t) && (!b) && (!lt) && (rt) && (rb) && (lb)) tile = 37;
	if ((!l) && (!r) && (!t) && (!b) && (lt) && (!rt) && (rb) && (lb)) tile = 38;
	if ((!l) && (!r) && (!t) && (!b) && (lt) && (rt) && (!rb) && (lb)) tile = 39;
	if ((!l) && (!r) && (!t) && (!b) && (lt) && (rt) && (rb) && (!lb)) tile = 40;
	if ((!l) && (!r) && (!t) && (!b) && (!lt) && (rt) && (!rb) && (lb)) tile = 41;
	if ((!l) && (!r) && (!t) && (!b) && (lt) && (!rt) && (rb) && (!lb)) tile = 42;
	if ((l) && (!r) && (t) && (!b) && (rb)) tile = 43;
	if ((!l) && (r) && (t) && (!b) && (lb)) tile = 44;
	if ((l) && (!r) && (!t) && (b) && (rt)) tile = 45;
	if ((!l) && (r) && (!t) && (b) && (lt)) tile = 46;

	debug("map type = " << type << ", tile = " << tile);
	map[x][y]->set_tile(tile);

	// if this tile doesn't exist set the standard one
	if (map[x][y]->get_picture() == 0)
	{
		debug("tile empty -> assigning standard tile");
		map[x][y]->set_tile(5);
	}

	// now give me a random variation
	map[x][y]->random_variation();
}

/* Do all transitions */

void Mapdef::smooth_all()
{
	debug("smooth_all()");

	for (int i = 1; i < 99; i++)
		for (int j = 1; j < 99; j++)
		{
			smooth(j, i);
		}
}

std::string Mapdef::extract_mapset(std::string filename)
{
	debug("extract_mapset()");

	/*P
	XMLTree doc(filename);	// Put the XML file into the tree
	XMLNode* top = doc.root();		// get the top node
	return top->property("Mapsetname")->value();
	*/
	return "default";
}

void Mapdef::load_from_mapstring(std::string mapstring)
{
	/*P
	for (int i = 0; i < 100; i++)
		for (int j = 0; j < 100; j++)
		{
			string temp;
			temp = mapstring[i*100+j];
			int type = temp.toInt();
			map[i][j] = new Maptile(i, j, type);
		}
	*/
}

void Mapdef::load(std::string filename)
{
	/*
	QFile f(filename);
	f.open(IO_ReadOnly);

	QDomDocument doc("Mapset");
	doc.setContent(&f);
	f.close();

	QDomElement map = doc.documentElement();
	mapdef->set_name(map.attribute("Name"));
	mapdef->set_info(map.attribute("Info"));
	mapdef->load_from_mapstring(map.attribute("Mapstring"));	

	for (QDomNode n = map.namedItem("Cities").firstChild(); !n.isNull(); n = n.nextSibling())
	{
		QDomElement e = n.toElement();
		City* city = new City(canvas, e.attribute("X").toInt(),
			      e.attribute("Y").toInt(), neutral, e.attribute("Name"),
			      e.attribute("Gold").toInt());

		city->set_armytype(0, e.attribute("Armytype_1").toInt());
		city->set_armytype(1, e.attribute("Armytype_2").toInt());
		city->set_armytype(2, e.attribute("Armytype_3").toInt());
		city->set_armytype(3, e.attribute("Armytype_4").toInt());

		citylist.append(city);
	}

	for (QDomNode n = map.namedItem("Ruins").firstChild(); !n.isNull(); n = n.nextSibling())
	{
		QDomElement e = n.toElement();
		ruinlist.append(new Ruin(canvas, e.attribute("X").toInt(),
				       e.attribute("Y").toInt(), e.attribute("Name")));
	}

	for (QDomNode n = map.namedItem("Temples").firstChild(); !n.isNull(); n = n.nextSibling())
	{
		QDomElement e = n.toElement();
		templelist.append(new Temple(canvas, e.attribute("X").toInt(),
				       e.attribute("Y").toInt(), e.attribute("Name")));

	}
	*/
}

void Mapdef::load_mapset(std::string filename)
{
	debug("load_mapset()");
	debug("filename = " << filename);

	char name[40];
	char fname[120];
	int r, g, b, moves, defense;
	unsigned int map_type;

	//Open file
	std::ifstream data_file(filename.c_str());

	//Get data
	int counter;
	data_file >> counter;
	for(int i = 0; i < counter; i++)
	{
		Mapset_tile* m = new Mapset_tile();
		data_file >> name;
		m->set_name(name);
		data_file >> name;		// name of the picture
		data_file >> moves;
		m->set_moves(moves);
		data_file >> defense;
		m->set_defense(defense);
		data_file >> r;
		data_file >> g;
		data_file >> b;
		SDL_Color color;
		color.r = r;
		color.g = g;
		color.b = b;
		m->set_color(color);
		data_file >> map_type;
		m->set_type(map_type);
		mapset->push_back(m);

		// load all tiles of this type
		for (int j = 0; j < 47; j++)
		{
			// and all variations of this tile
			for (int k = 0; k < 6; k++)
			{
				sprintf(fname, "../pic/map/default/%s_%i_%i.png", name, j, k);

				SDL_Surface* newpic = IMG_Load(fname);
				if (newpic != 0)
				{
					m->set_max_variations(j, k);
					
				}
				m->set_pixmap(j, k, newpic);
			}
		}

		//m->print_debug_info();	// Print debugging information
	}
}
