//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MAPSET_TILE_H
#define MAPSET_TILE_H

#include <SDL.h>
#include <string>
#include <vector>

class Mapset_tile
{
	public:
		// CREATORS
		Mapset_tile();
		~Mapset_tile();

		// MANIPULATORS
		void set_name(std::string name_){name = name_;}
		void set_moves(int moves_){moves = moves_;}
		void set_defense(int defense){d_defense = defense;}
		void set_max_variations(int index, int max_variations_)
			{max_variations[index] = max_variations_;}
		void set_color(SDL_Color& color_){color = color_;}
		void set_pixmap(int t, int v, SDL_Surface* pixmap_){pixmap[t][v] = pixmap_;}
		void set_type(unsigned int type_) {type = type_;}
		void print_debug_info();

		// ACCESSORS
		SDL_Surface* get_pixmap(int t, int v){return pixmap[t][v];}
		std::string get_name(){return name;}
		int get_moves(){return moves;}
		int get_defense(){return d_defense;}
		int get_max_variations(int i){return max_variations[i];}
		SDL_Color get_color(){return color;}
		unsigned int get_type(){return type;}
				
	private:
		// DATA
		SDL_Surface* pixmap[47][6];	// picture
		std::string name;		// name	
		int max_variations[47];		// number of max. variations for all tiles
		int moves;			// moves needed to walk over maptile
		int d_defense;			// defense bonus
		SDL_Color color;		// color shown in the smallmap
		unsigned int type;
};

extern std::vector<Mapset_tile*>* mapset;

#endif // MAPSET_TILE_H
