//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef PATH_H
#define PATH_H

#include <set>
#include <map>
#include <list>
#include <function.h>
#include <paragui.h>

class PathItem;
class Stack;

class Path : public std::list<PG_Point*>
{
	public:
		Path();
		~Path();

		//Set methods
		void set_heuristic (double value);
			//the higher the heuristic the more straightforward
			//the path, the lower the heuristic the more time
			//the algortihm needs and the better paths it finds

		//Get methods
		double get_heuristic();
		
		//for calculating the path
		bool calculate(Stack* s, PG_Point _dest);
		bool check_path(Stack* s, bool recalculate = true);

		//use these functions to delete items in the list (pointers!!)
		Path::iterator fl_erase(Path::iterator it);
		void fl_clear();

	private:
		bool calculate_path(Stack* s, PG_Point _dest, PG_Point start,
				std::list<PG_Point>* pathlist,
				bool write_directly = false);
		
		void check_surroundings(PathItem* p);
		bool is_blocked (int x, int y);
		PathItem create_item (int x, int y, PathItem* parent);
		void insert_item (PathItem item);

		//the rest is purely for calculating the path

		double d_heuristic;
		PG_Point d_dest;
		Stack* d_unit;	//need this for is_blocked()
		unsigned int d_stack_bonus, d_army_bonus;
		bool d_has_ship, d_has_land;
			//has_land refers to units who can't cross water
		std::multiset<PathItem, std::less<PathItem> > d_openlist;
		std::map<unsigned int, PathItem, std::less<unsigned int> > d_closelist;
			//the key is built like key = (x << 16) + y
			//helps with searching (after some cycles closelist
			//contains by far most elements), you only need to find
			//keys Disadvantage: doesn't work on <32bit machines :)
};

#endif // PATH_H
