//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef PLAYER_H
#define PLAYER_H

#include <string>
#include <list>
#include <pgmessageobject.h>
#include "defs.h"
#include "fight.h"
#include "action.h"
#include "city.h"
#include "armyset.h"

class Player : public PG_MessageObject
{
	public:
		// CREATORS
		Player (std::string name, std::string armyset, SDL_Color color);
		Player(const Player&);
		~Player();
		
		// MANIPULATORS
		void set_armyset(std::string armyset){d_armyset = armyset;}
		void set_dead(bool dead){d_dead = dead;}
//NIY		Director* set_director (Director* new_master);
    	void set_computer(bool computer){d_computer = computer;}
		void addGold(int gold);
		void withdrawGold(int gold);
		void clear_actionlist();
	    std::string getAICommand();
        void computerTurn(); 
		
		// ACCESSORS
		bool isComputer(){return d_computer;}
		std::list<Action*> get_actionlist(){return d_actions;}
		Armyset* get_armyset();
		std::string get_armyset_name(){return d_armyset;}
		SDL_Color get_color() const {return d_color;}
		int getGold(){return d_gold;}
		std::string get_name(){return d_name;}
		void dump_actionlist();
		
		//ACTIONS
		bool start_turn();
		bool next_turn();
		
		bool stack_split(Stack* s);
		bool stack_join(Stack* receiver, Stack* joining);

		bool stack_move(Stack* s, PG_Point dest, bool follow);
		
		Fight::RESULT stack_fight(Stack* sattacker, Stack* sdefender);
		Fight::RESULT stack_fight_city(std::vector<Stack*> attackers,
						City* target);

		bool stack_search_ruin(Stack* s, Ruin* r);
		bool stack_visit_temple(Stack* s, Temple* t);

		bool city_occupy(City* c);
		bool city_raze(City* c);

		bool city_upgrade_defense(City* c);
		bool city_buy_production(City* c, int armytype);
		bool city_change_production(City* c, int production);
		
	private:
		bool give_reward (int level, Stack* s, fl_Treasure* reward = 0);
			//if reward != 0, give this reward, else randomize
		bool stack_move_one_step(Stack* s);
		bool enough_moves(Stack* s);

		// DATA
		SDL_Color d_color;
		std::string d_name;
		std::string d_armyset;	// name of the armyset
		int d_gold;
		bool d_computer;	// is this an AI player?
		bool d_dead;		// is player already dead?

		//Director* d_master	//the entity which controls the player
					//(NIY)
		std::list<Action*> d_actions;	//the list of the actions done
						//by the player
};

#endif // PLAYER_H
