//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "playerlist.h"
#include <pgmessagebox.h>
#include "citylist.h"
#include "player.h"
#include "defs.h"

#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

Playerlist* playerlist;

Playerlist::Playerlist()
	:d_activeplayer(0)
{
	SDL_Color gray;
	gray.r = gray.g = gray.b = 150;
	d_neutral = new Player("Neutral", "default", gray);
}

Playerlist::~Playerlist()
{
}

void Playerlist::check_players()
{
	debug("check_players()");
	iterator it = begin ();
	while (it != end ())
	{
		if (!citylist->count_cities(&(*it)))
		{
			char buffer[100];
			sprintf(buffer, "Player %s died", (*it).get_name().c_str());

			PG_MessageBox mb(0, PG_Rect(20, 20, 200, 150), "Player died", buffer,
					PG_Rect(20, 110, 80, 30), "OK");

			mb.Show();
			mb.WaitForClick();
			mb.Hide();

			iterator nextit = it;
			nextit++;
			erase(it);			// remove player from list

			emit(MSG_PLAYERLIST_PLAYER_DEAD);

			// only 1 player left ?
			if (size() == 1)
			{
				sprintf(buffer, "Player %s won", (*begin()).get_name().c_str());

				PG_MessageBox mb2(0, PG_Rect(20, 20, 200, 150), "Player won", buffer,
						PG_Rect(20, 110, 80, 30), "OK");

				mb2.Show();
				mb2.WaitForClick();
				mb2.Hide();
				
				cout << "or still here?" << endl;
				emit(MSG_PLAYERLIST_GAME_WON);
			}
			it = nextit;	// do this at the end to catch abuse of invalid it
		} else {
			++it;
		}
	}
}

void Playerlist::next_player()
{
	debug("next_player()");

	if (!d_activeplayer)
	{
		d_activeplayer = &(*begin());
		emit(MSG_PLAYERLIST_NEXT_TURN);
		d_activeplayer->start_turn();
		return;
	}
	else
	{
		for (iterator it = begin(); it != end(); ++it)
		{
			if (&(*it) == d_activeplayer)
			{
				if (++it == end())
				{
					it = begin();
				}
				d_activeplayer = &(*it);
				emit(MSG_PLAYERLIST_NEXT_TURN);
				d_activeplayer->start_turn();
				return;
			}
		}
	}
	assert(0);
}

Player* Playerlist::get_by_name(std::string name)
{
	debug("get_by_name()");
	if (name == "Neutral") return d_neutral;
	else
	{
		for (iterator it = begin(); it != end(); ++it)
		{
			if ((*it).get_name() == name) return &(*it);
		}
	}
	return 0;
}

int Playerlist::get_nr_of_player(Player* p)
{
	int index = 0;
	for (iterator it = begin(); it != end(); ++it, ++index)
	{
		if (&(*it) == p) return index;
	}
	return -1;	
}
