//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "stack.h"
#include "player.h"
#include "mapdef.h"
#include "path.h"

//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

Stack::Stack(Player* player, PG_Point pos)
	:Object(pos), d_player(player), d_defending(false)
{
	d_path = new Path();
	static unsigned int counter = 0;
	d_id = (++counter);
}

Stack::Stack(Stack& s)
	:Object(s.d_pos), d_id(s.d_id), d_player(s.d_player),
	d_defending(s.d_defending)
{
	clear();
	d_path = new Path();
	//deep copy the other stack's armies
	for (iterator sit = s.begin(); sit != s.end(); sit++)
	{
		Army* a;
		a = new Army((**sit));
		push_back(a);
	}
}

Stack::~Stack()
{
  	debug("~Stack()");
	delete d_path;
	fl_clear();
}

// This evaluation should be in the path class! It should
// calculate to each point if it the stack has still enough moves

bool Stack::enough_moves_next_step()
{
	debug("enough_moves_next_step()");

	unsigned int map_type = mapdef->map[d_path->front()->x][d_path->front()->y]->get_maptile_type();
	unsigned int stack_bonus = 0xFFFFFFFF;
	int needed_moves = 0;
	
	for (iterator it = begin(); it != end(); it++)
	{
		stack_bonus = stack_bonus & (*it)->get_move_bonus();
	}
	
	if (stack_bonus & map_type) needed_moves = 2;
	else needed_moves = mapdef->map[d_path->front()->x][d_path->front()->y]->get_moves();
	
	debug(needed_moves);
	
	if ((get_group_moves() >= needed_moves)) return true;

	return false;
}

bool Stack::move_one_step()
{
	debug("move_one_step()");

	if (!enough_moves_next_step()) return false;

	d_pos = **d_path->begin();
	
	//DI why does stack know anything about the mapdef?? move this to bigmap
	decrement_moves(mapdef->map[d_pos.x][d_pos.y]->get_moves());

	//now remove first point of the path
	d_path->fl_erase(d_path->begin());

	return true;
}

int Stack::get_group_moves()
{
	// Set maximum moves to 100
	int minimum_moves = (*begin())->get_moves();

	// Get moves of whole group and set max to minimum
	for (iterator it = begin(); it != end(); it++) 
	{
		if ((*it)->get_moves() < minimum_moves) 
			minimum_moves = (*it)->get_moves();
	}

        return minimum_moves;  
}

// decrement each armys moves by needed moves to travel

void Stack::decrement_moves(int moves)
{
	debug("decrement_moves()");

	for (iterator it = begin(); it != end(); it++) 
	{
		(*it)->decrement_moves(moves);
	}
}

// Purpose: Return the strongest army of a group
// Note: If a hero is present return it. If there are two similar armies
// (two of the same strength, or two heroes) return the first in the sequence.

Army* Stack::get_strongest_army() const
{
	int highest_strength = 0;

	for (const_iterator it = begin(); it != end(); ++it) 
	{
		// if hero
		if (((*it)->get_type() == 0) || ((*it)->get_type() == 1))
		{
			return *it;
		}
		else if ((*it)->get_strength() > highest_strength) 
		{
			highest_strength = (*it)->get_strength();
			return *it;
		}
	}
	return 0;
}

bool Stack::has_hero() const
{
	Army* a = get_strongest_army();
	assert(a);
	if ((a->get_type() == 0) || (a->get_type() == 1)) return true;
	else return false;
}

void Stack::bless()
{
	for (iterator it = begin(); it != end(); it++)
	{
		(*it)->bless();
	}
}

Army* Stack::get_first_ungrouped_army() const
{
	for (const_iterator it = begin(); it != end(); it++) 
	{
		if (!(*it)->is_grouped())
		{
			return *it;
		}
	}
	return 0;
}

bool Stack::is_friend(Player* player) const
{
	return d_player == player;
}

void Stack::nextTurn()
{
	d_defending = false;
	for (iterator it = begin(); it != end(); it++) 
	{
		(*it)->reset_moves();
		// TODO: should be moved in a more appropriate place => class Player
		d_player->withdrawGold((*it)->get_upkeep());
		(*it)->heal();
	}
}

void Stack::fl_clear()
{
	for (iterator it = begin(); it != end(); it++)
		delete (*it);
	clear();
}

Stack::iterator Stack::fl_erase(Stack::iterator object)
{
	delete (*object);
	return erase(object);
}

// End of file
