//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef STACK_H
#define STACK_H

#include <list>
#include <paragui.h>
#include "Object.h"
#include "army.h"
#include "path.h"

class Player;

class Stack : public Object, public std::list<Army*>
{
	public:
		// CREATORS
		Stack(Player* player, PG_Point pos);
		Stack(Stack& s);
		~Stack();

		// MANIPULATORS
		void set_defending(bool defending){d_defending = defending;}
		void nextTurn();
		void decrement_moves(int moves);
		bool enough_moves_next_step();
		bool move_one_step();
		int get_group_moves();
		void reset_moves();
		void upkeep();
		void bless();

		// ACCESSORS
		unsigned int get_id() const {return d_id;}
		Player* get_player() const {return d_player;}
		Path* get_path() const {return d_path;}
		PG_Point next_step();
		Army* get_strongest_army() const;
		Army* get_first_ungrouped_army() const;
		bool has_hero() const;
		bool is_enemy(Player* player) const;
		bool is_friend(Player* player) const;
		bool get_defending() const {return d_defending;}

		//MISCELLANEOUS
		void fl_clear();
		iterator fl_erase(iterator object);
		
	private:	
		// DATA
		unsigned int d_id;
		Player* d_player;
		Path* d_path;
		bool d_defending;
};

#endif // STACK_H
