//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "stacklist.h"
#include <limits.h>
#include "stack.h"
#include "city.h"
#include "army.h"
#include "path.h"
#include "citylist.h"
#include "playerlist.h"
#include "../graphic/FightDialog.h"
#include "defs.h"
#include <algorithm>

#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

Stacklist* stacklist;

Stacklist::Stacklist()
	:d_activestack(0)
{
}

Stacklist::~Stacklist()
{
}

// Get next Stack of the same player and set it as d_activestack

Stack* Stacklist::set_next()
{	
	debug("set_next()");

	// set the iterator to the activestack 
	for (const_iterator it = begin(); it != end(); ++it)
	{
		if ((*it) != d_activestack && (*it)->get_player() == playerlist->get_activeplayer() && !(*it)->get_defending())
		{
			d_activestack = *it;
			return d_activestack;
		}
	}
	return 0;
}

void Stacklist::nextTurn()
{
	debug("next_turn()");

	for (iterator it = begin(); it != end(); it++)
	{
		(*it)->nextTurn();
	}
}

Fight::RESULT Stacklist::fight(Stack* attacker, std::vector<Stack*>* defender)
{
	//the types here look stupid, but this function is going to be
	//obsoleted soon, so I don't put much interest in it
	debug("fight()");
	assert (attacker && defender);

	std::vector<Stack*> v(1);
	v[0] = attacker;

	FightDialog fightdialog(v, (*defender), 0);
	fightdialog.Show();
	fightdialog.battle();

	Fight::RESULT result = fightdialog.get_result();

	//remove stiffs in attacker army
	for (Stack::iterator it = attacker->begin(); it != attacker->end();)
	{
		if ((*it)->get_hp() <= 0)
		{
			it = attacker->fl_erase(it);
			continue;
		}
		it++;
	}
	if (attacker->empty())
	{
		fl_remove(attacker);
		attacker= 0;
	}

	//and in the defender armies, too
	for (unsigned int i = 0; i < defender->size(); i++)
	{
		for (Stack::iterator it = (*defender)[i]->begin();
				it != (*defender)[i]->end();)
		{
			if ((*it)->get_hp() <= 0)
			{
				it = (*defender)[i]->fl_erase(it);
				continue;
			}
			it++;
		}
		if ((*defender)[i]->empty())
		{
			fl_remove((*defender)[i]);
			(*defender)[i] = 0;
		}
	}

	return result;
}

Stack* Stacklist::get_object_at(int x, int y) const
{
	for (const_iterator it = begin(); it != end(); it++)
	{
		if (((*it)->get_pos().x == x) && ((*it)->get_pos().y == y))
		{
			return *it;
		}
	}
	return 0;
}

Stack* Stacklist::get_object_at(PG_Point point) const
{
	return get_object_at(point.x, point.y);
}

void Stacklist::move(Stack* stack)
{
	debug("move()");

	while (stack->get_path()->size() > 1)
	{
		if (!stack->move_one_step()) return;
		emit(MSG_STACKLIST_STACK_MOVED);
	}

	if (stack->get_path()->size() == 1 && stack->enough_moves_next_step())
	{
		PG_Point pos = **stack->get_path()->begin();
		Stack* target = get_object_at (pos);
		City* city = citylist->get_object_at(pos);

		// fight city comes first to avoid overlapping with fight army
		if ((city) && (city->get_player() != stack->get_player()))
		{
			std::vector<Stack*> attackervector;
			attackervector.push_back(stack);

			if (!defenders_in_city(city).empty())
			{
				Fight::RESULT result = playerlist->get_activeplayer()->stack_fight_city(attackervector, city);

				if (result == Fight::DEFENDER_WON)
				{
					emit(MSG_STACKLIST_STACK_DIED);
				}
				else if (result == Fight::ATTACKER_WON)
				{
					stack->move_one_step();
					emit(MSG_STACKLIST_CITY_OCCUPIED);
				}
				else emit(MSG_STACKLIST_REDRAW);
				return;
			}
			// no defender present
			else
			{
				stack->move_one_step();
				city->set_player(stack->get_player());
				emit(MSG_STACKLIST_CITY_OCCUPIED);
				return;
			}
		}
		else if (target)
		{
			if (target->get_player() == stack->get_player())
			{
				stack->move_one_step();
				playerlist->get_activeplayer()->stack_join(stack, target);
				emit(MSG_STACKLIST_STACK_MOVED);
				return;
			}
			else
			{
				//dirty hack, but this function is becoming
				//obsolete, too
				Fight::RESULT result = playerlist->get_activeplayer()->stack_fight(stack, target);

				if (result == Fight::DEFENDER_WON)
				{
					emit(MSG_STACKLIST_STACK_DIED);
				}
				else if (result == Fight::ATTACKER_WON)
				{
					stack->move_one_step();
					emit(MSG_STACKLIST_STACK_MOVED);
				}
				else emit(MSG_STACKLIST_REDRAW);
				return;
			}
		}
		else
		{
			stack->move_one_step();
			emit(MSG_STACKLIST_STACK_MOVED);
		}
	}
}

std::vector<Stack*> Stacklist::defenders_in_city(City *city)
{
	debug("defenders_in_city()");  

	std::vector<Stack*> stackvector;
	PG_Point pos = city->get_pos();

	for (int i = pos.x; i < pos.x + 2; i++)
	{
		for (int j = pos.y; j < pos.y + 2; j++)
		{
			Stack* stack = get_object_at(i, j);
			if (stack)
				stackvector.push_back(stack);
		}
	}

	return stackvector;
}

Stack* Stacklist::get_ambiguity(Stack* s)
//We only expect one ambiguity at a time with stacks of the same player. This
//never happens except when a stack comes to halt on another stack during
//long movements
{
	for (iterator it = begin(); it != end(); it++)
		if ((s->get_pos().x == (*it)->get_pos().x)
			&& (s->get_pos().y == (*it)->get_pos().y))
			return (*it);
	
	return 0;
}

PARAGUI_CALLBACK(Stacklist::remove_all)
{
	Player* player = (Player*)clientdata;

	for (iterator it = begin(); it != end(); it++)
	{
	  if ((*it)->get_player() == player) stacklist->remove(*it);
	}
	return true;
}

void Stacklist::fl_clear()
{
	for (iterator it = begin(); it != end(); it++)
	{
		delete (*it);
	}
	
	clear();
}

Stacklist::iterator Stacklist::fl_erase(iterator object)
{
	delete (*object);
	
	return erase(object);
}

bool Stacklist::fl_remove(Stack* object)
{
	debug("removing stack with id "<<object->get_id()<<endl)
	iterator stackit = std::find(begin(), end(), object);
	if (stackit != end())
	{
		delete object;
		erase(stackit);
		return true;
	}
	return false;
}
