//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef STACKLIST_H
#define STACKLIST_H

#include <list>
#include <pgmessageobject.h>
#include <pgeventobject.h>
#include "fight.h"

class City;
class Stack;

class Stacklist : public std::list<Stack*>, public PG_MessageObject, public PG_EventObject
{
	public:
		// CREATORS
		Stacklist();
		~Stacklist();

		// MANIPULATORS
		void set_activestack(Stack* activestack){d_activestack = activestack;}
		Stack* set_next();
		void move(Stack* s);
		Fight::RESULT fight(Stack* attacker, std::vector<Stack*>* defender);
		bool can_join(Stack* s, Stack* d);
		void nextTurn();		// do all stuff needed to proceed to next turn
		PARAGUI_CALLBACK(remove_all);	// remove all stacks of this player
		std::vector<Stack*> defenders_in_city(City* c);
		
		// ACCESSORS
		Stack* get_activestack() const {return d_activestack;}
		Stack* get_object_at(int x, int y) const;
		Stack* get_object_at(PG_Point point) const;
		Stack* get_ambiguity(Stack* s);
			//look if another stack occupies the same point as s
			//(need to force joining of stacks when moving)

		//remove methods which also frees the memory of the pointers
		void fl_clear();
		iterator fl_erase(iterator object);
		bool fl_remove(Stack* object);

	private:
		Stack* d_activestack;
};

extern Stacklist* stacklist;

#endif // STACKLIST_H
