//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <iostream.h>
#include <qpoint.h>
#include "game_scenario.h"
#include "turninfo.h"
#include "citylist.h"
#include "playerlist.h"
#include "stacklist.h"
#include "ruinlist.h"
#include "templelist.h"

//#define DEBUG_IT

#ifdef DEBUG_IT
	#define debug(x) {x;}
#else
	#define debug(x)
#endif

#define cdbg cerr<<__FILE__<<": "<<__LINE__<<": "

Turninfo* turninfo;

Turninfo::Turninfo()
{
	f = new QFile("turninfo.dat");
	t = new QTextStream(f);
}

Turninfo::~Turninfo()
{
}


void Turninfo::close()
{
	debug(cdbg << "close()\n");
	f->close();
}

void Turninfo::parse()
{
	debug(cdbg << "parse()\n");

	// create all lists anew
	playerlist.clear();
	citylist.clear();
	stacklist.clear();
	ruinlist.clear();
	templelist.clear();
	
	if (!f->open(IO_ReadOnly)) cerr << "couldn't open turninfo.dat\n";

	QString str;
	while (!t->eof())
	{
		str = t->readLine();
		if (str == "CITY")
		{
			str = t->readLine();
			int x = str.toInt();
			str = t->readLine();
			int y = str.toInt();
			str = t->readLine();
			QString name = str;
			str = t->readLine();
			Player* player = playerlist.get_by_name(str);
			str = t->readLine();
			int gold = str.toInt();
			str = t->readLine();
			int defense_level = str.toInt();
			str = t->readLine();
			int duration = str.toInt();
			str = t->readLine();
			int production = str.toInt();
			str = t->readLine();
			bool burnt = str.toInt();
			City* city = new City(canvas_bigmap, x, y, player, name, gold);
			for (int i = 0; i < 4; i++)
			{
				str = t->readLine();
				int nr = str.toInt();
				city->set_armytype(i, nr);
			}
			city->set_defense_level(defense_level);
			city->set_duration(duration);
			city->set_production(production);
			city->set_burnt(burnt);
			citylist.append(city);
		}

		if (str == "RUIN")
		{
			str = t->readLine();
			QString name = str;
			str = t->readLine();
			int x = str.toInt();
			str = t->readLine();
			int y = str.toInt();
			str = t->readLine();
			bool searched = str.toInt();
			Ruin* ruin = new Ruin(canvas_bigmap, x, y, name, searched);
			ruinlist.append(ruin);
		}

		if (str == "TEMPLE")
		{
			str = t->readLine();
			QString name = str;
			str = t->readLine();
			int x = str.toInt();
			str = t->readLine();
			int y = str.toInt();
			Temple* temple = new Temple(canvas_bigmap, x, y, name);
			templelist.append(temple);
		}

		if (str == "PLAYER")
		{
			str = t->readLine();
			Player* player = new Player(str);
			str = t->readLine();
			int r = str.toInt();
			str = t->readLine();
			int g = str.toInt();
			str = t->readLine();
			int b = str.toInt();
			player->color.setRgb(r, g, b);
			str = t->readLine();
			int gold = str.toInt();
			player->set_gold(gold);
			str = t->readLine();
			int computer = str.toInt();
			player->set_computer(computer);
			str = t->readLine();
			bool dead = str.toInt();
			player->set_dead(dead);
			playerlist.append(player);
		}

		if (str == "STACK")
		{
			str = t->readLine();
			Player* player = playerlist.get_by_name(str);
			str = t->readLine();
			int x = str.toInt();
			str = t->readLine();
			int y = str.toInt();
			str = t->readLine();
			QList<QPoint> path;
			int defending;
			if (str == "PATH")	// it HAS to be PATH else shit happened
			{
				str = t->readLine();
				while (str != "PATHEND")
				{
					int x_path = str.toInt();
					str = t->readLine();
					int y_path = str.toInt();
					path.append(new QPoint(x_path, y_path));
					str = t->readLine();
				}
				str = t->readLine();
				defending = str.toInt();

			}
			else defending = str.toInt();

			Stack* stack = new Stack(player, QPoint(x, y));
			stack->set_path(path);
			stack->set_defending(defending);
			stacklist.append(stack);
			
			while (str != "STACKEND")
			{
				str = t->readLine();
				if (str == "ARMY")	// if not shit happened
				{
					str = t->readLine();
					QString name = str;
					str = t->readLine();
					int max_HP = str.toInt();
					str = t->readLine();
					int cur_HP = str.toInt();
					str = t->readLine();
					int strength = str.toInt();
					str = t->readLine();
					int moves = str.toInt();
					str = t->readLine();
					int upkeep = str.toInt();
					str = t->readLine();
					int production = str.toInt();
					str = t->readLine();
					int production_cost = str.toInt();
					str = t->readLine();
					int type = str.toInt();
					str = t->readLine();
					int grouped = str.toInt();
					str = t->readLine();
					int defence = str.toInt();
					str = t->readLine();
					int vitality = str.toInt();
					str = t->readLine();
					int blessed = str.toInt();

					Army* a = new Army(name, max_HP, cur_HP, strength,
						      moves, upkeep, production, production_cost,
						      type, grouped, defence, vitality, MAPTILE_NONE, blessed);

					a->set_armyset_army(::get_armyset("default"), type);

					stack->army.append(a);
				}
			}
		}
	}
	f->remove();
}

void Turninfo::create()
{
	if (f->open(IO_WriteOnly))
	{
		QString str;
		str = "TURNINFO\n";
		str += playerlist.get_info();
		str += citylist.get_info();
		str += stacklist.get_info();
		str += ruinlist.get_info();
		str += templelist.get_info();
		str += "END\n";
		*t << str;
		f->close();
	}
	else cerr << "couldn't open turninfo.dat\n";
}

void Turninfo::remove()
{
	f->remove();
}
