//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "ArmyInfo.h"
#include <pglabel.h>
#include "army.h"

//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

ArmyInfo::ArmyInfo(Army* army, bool pressable, PG_Widget* parent, PG_Rect rect)
	:PG_ThemeWidget(parent, rect, true), d_army(army), d_pressed(false),
        d_pressable(pressable)
{
	d_army->is_grouped() ? d_pressed = true : d_pressed = false;

	// show number of moves
	char buffer[80];
	sprintf(buffer, "%i", d_army->get_moves());
	d_l_moves = new PG_Label(this, PG_Rect(13, 43, 20, 20), buffer);

	Show();
}

ArmyInfo::~ArmyInfo()
{
	delete d_l_moves;
}

void ArmyInfo::eventDraw(SDL_Surface* surface, const PG_Rect& rect)
{
	debug("eventDraw()");
	PG_ThemeWidget::eventDraw(surface, rect);

	assert(d_army);

	DrawBorder(rect, 1, !d_pressed);

	// show pic of army in the button
	SDL_BlitSurface(d_army->get_pixmap(), NULL, surface, PG_Rect(2, 2, 40, 40).SDLRect());

	// draw the hitpoints-bar
	SDL_Color red = {255, 0, 0};
	SDL_Color green = {0, 255, 0};

	float factor = (float)d_army->get_hp() / (float)d_army->get_max_hp() * 38;
	DrawLine(4, 64, 38, 64, red, 5);
	DrawLine(4, 64, factor, 64, green, 5);
}

bool ArmyInfo::eventMouseButtonDown(const SDL_MouseButtonEvent* event)
{
	debug("eventMouseButtonDown()");
        if (!d_pressable) return true;
	if (d_army == 0) return true;

	if (d_pressed) 
	{
		d_pressed = false;
		d_army->set_grouped(false);
	}
	else
	{
		d_pressed = true;
		d_army->set_grouped(true);
	}

	Redraw();
	return true;
}
