//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "MainWindow.h"
#include "w_edit.h"
#include "defs.h"
#include "Configuration.h"
//#include "citiesreport.h"

#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

MainWindow::MainWindow(GameScenario* gameScenario, const PG_Rect& rect)
	:PG_ThemeWidget(0, rect, true)
{
	// create menus
	d_fileMenu = new PG_PopupMenu(0, 20, 0, 0);
	d_fileMenu->addMenuItem("Load...", 0, MSG_CALLBACK_OBJ(&MainWindow::file_load), this);
	d_fileMenu->addMenuItem("Save", 1, MSG_CALLBACK_OBJ(&MainWindow::file_save), this);
	d_fileMenu->addMenuItem("Save as...", 2, MSG_CALLBACK_OBJ(&MainWindow::file_saveas), this);
	d_fileMenu->addSeparator();
	d_fileMenu->addMenuItem("Quit", 3, MSG_CALLBACK_OBJ(&MainWindow::file_quit), this);

	d_reportsMenu = new PG_PopupMenu(0, 170, 0, 0);
	d_reportsMenu->addMenuItem("Armies...", 0, MSG_CALLBACK_OBJ(&MainWindow::reports_armies), this);
	d_reportsMenu->addMenuItem("Cities...", 1, MSG_CALLBACK_OBJ(&MainWindow::reports_cities), this);
	d_reportsMenu->addMenuItem("Gold...", 2, MSG_CALLBACK_OBJ(&MainWindow::reports_gold), this);
	
	d_optionsMenu = new PG_PopupMenu(0, 0, 0, 0);
	d_optionsMenu->addMenuItem("Smooth scrolling", 0, MSG_CALLBACK_OBJ(&MainWindow::options_smooth_map_scrolling), this);
	d_optionsMenu->addMenuItem("Show next player popup", 0, MSG_CALLBACK_OBJ(&MainWindow::options_showNextPlayer), this);

	d_helpMenu = new PG_PopupMenu(0, 470, 0, 0);
	d_helpMenu->addMenuItem("Help", 0, MSG_CALLBACK_OBJ(&MainWindow::help_help), this);
	d_helpMenu->addSeparator();
	d_helpMenu->addMenuItem("About...", 1, MSG_CALLBACK_OBJ(&MainWindow::help_about), this);

	// create menubar
	d_menuBar = new PG_MenuBar(this, PG_Rect(0, 0, rect.w, 25));
	d_menuBar->Add("File", d_fileMenu);
	d_menuBar->Add("Reports", d_reportsMenu);
	d_menuBar->Add("Options", d_optionsMenu);
	d_menuBar->Add("Help", d_helpMenu);
	
	d_w_edit = new W_Edit(gameScenario, this, PG_Rect(rect.x, rect.y + 25, rect.w, rect.h - 25));

	d_fileMenu->Hide();
	d_reportsMenu->Hide();
	d_optionsMenu->Hide();
	d_helpMenu->Hide();
}

MainWindow::~MainWindow()
{
	delete d_menuBar;
	delete d_fileMenu;
	delete d_reportsMenu;
	delete d_optionsMenu;
	delete d_helpMenu;
	delete d_w_edit;
}

void MainWindow::start_game()
{
	d_w_edit->start_game();
}

PARAGUI_CALLBACK(MainWindow::file_load)
{
	std::cerr << "not implemented yet\n";
	return true;
}

PARAGUI_CALLBACK(MainWindow::file_save)
{
	std::cerr << "not implemented yet\n";
	return true;
}

PARAGUI_CALLBACK(MainWindow::file_saveas)
{
	std::cerr << "not implemented yet\n";
	return true;
}

PARAGUI_CALLBACK(MainWindow::file_quit)
{
	emit(MSG_MAINWINDOW_QUIT);
	return true;
}

PARAGUI_CALLBACK(MainWindow::reports_armies)
{
	std::cerr << "not implemented yet\n";
	return true;
}

PARAGUI_CALLBACK(MainWindow::reports_cities)
{
	std::cerr << "not ported yet\n";
	/*
	   CitiesReport *report = new CitiesReport();
	   report->exec();
	   delete report;
	   */
	return true;
}

PARAGUI_CALLBACK(MainWindow::reports_gold)
{
	std::cerr << "not ported yet\n";
	/*
	   GoldReport *report = new GoldReport();
	   report->exec();
	   delete report;
	   */
	return true;
}

PARAGUI_CALLBACK(MainWindow::options_smooth_map_scrolling)
{
	Configuration::smoothScrolling = !Configuration::smoothScrolling;
	return true;
}

PARAGUI_CALLBACK(MainWindow::options_showNextPlayer)
{
	Configuration::showNextPlayer = !Configuration::showNextPlayer;
	return true;
}

PARAGUI_CALLBACK(MainWindow::help_help)
{
	std::cerr << "not implemented yet\n";
	return true;
}

PARAGUI_CALLBACK(MainWindow::help_about)
{
	std::cerr << "not implemented yet\n";
	return true;
}

// End of file
