//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "d_buy_production.h"
#include "army.h"
#include "armyset.h"
#include "city.h"
#include "player.h"

D_Buy_Production::D_Buy_Production(City* city, PG_Widget* parent, PG_Rect rect)
	:PG_Window(parent, rect, "Buy Production", true)
{
	d_city = city;
	d_armyset = city->get_player()->get_armyset();
	d_gold = city->get_player()->getGold();
	d_chosen_army = -1;

	int ca;		// current army
	int ci;		// current index

	for (int i = 0; i < 2; i++)
	{
		for (int j = 0; j < 5; j++)
		{
			ci = i*5 + j;
			ca = ci + 2;

			d_b_production[ci] = new PG_Button(this, ci, PG_Rect(20 + j*44, i*44 + 30, 44, 44));
			d_b_production[ci]->SetToggle(true);
			d_b_production[ci]->SetIcon(d_armyset->army[ca].get_pixmap(), 0, 0);
			d_b_production[ci]->SetEventObject(MSG_BUTTONCLICK, this,
					(MSG_CALLBACK_OBJ)&D_Buy_Production::b_production_clicked);

			// enough money to buy this production-type or have it already?
			if ((d_armyset->army[ca].get_production_cost() > d_gold) || (d_city->have_production(ca)))
				d_b_production[ci]->EnableReceiver(false);
		}
	}

	new PG_Label(this, PG_Rect(20, 130, 80, 20), "Name:");
	new PG_Label(this, PG_Rect(20, 150, 80, 20), "Duration:");
	new PG_Label(this, PG_Rect(20, 170, 80, 20), "Strength:");
	new PG_Label(this, PG_Rect(20, 190, 80, 20), "Moves:");
	new PG_Label(this, PG_Rect(20, 210, 80, 20), "Upkeep:");

	d_l_production = new PG_Label(this, PG_Rect(120, 130, 80, 20), "0");
	d_l_duration = new PG_Label(this, PG_Rect(120, 150, 80, 20), "0");
	d_l_strength = new PG_Label(this, PG_Rect(120, 170, 80, 20), "0");
	d_l_moves = new PG_Label(this, PG_Rect(120, 190, 80, 20), "0");
	d_l_upkeep = new PG_Label(this, PG_Rect(120, 210, 80, 20), "0");
	d_l_production_cost = new PG_Label(this, PG_Rect(120, 210, 80, 20), "0");
	d_b_buy = new PG_Button(this, 11, PG_Rect(260, 30, 60, 30), "Buy");
	d_b_cancel = new PG_Button(this, 11, PG_Rect(260, 70, 60, 30), "Cancel");

	d_b_buy->SetEventObject(MSG_BUTTONCLICK, this, (MSG_CALLBACK_OBJ)&D_Buy_Production::b_buy_clicked);
	d_b_cancel->SetEventObject(MSG_BUTTONCLICK, this, (MSG_CALLBACK_OBJ)&D_Buy_Production::b_cancel_clicked);
}

D_Buy_Production::~D_Buy_Production()
{
}

PARAGUI_CALLBACK(D_Buy_Production::b_buy_clicked)
{
	Hide();
	SendMessage(NULL, MSG_MODALQUIT, 0, 0);
	return true;
}

PARAGUI_CALLBACK(D_Buy_Production::b_cancel_clicked)
{
	d_chosen_army = -1;
	Hide();
	SendMessage(NULL, MSG_MODALQUIT, 0, 0);
	return true;
}

PARAGUI_CALLBACK(D_Buy_Production::b_production_clicked)
{
	d_chosen_army = id + 2;

	// deselect all other buttons
	for (int i = 0; i < 10; i++)
	{
		if (i != id) d_b_production[i]->SetPressed(false);
	}

	// selected armytype
	Armyset_army* a = &d_armyset->army[d_chosen_army];

	char buffer[40];
	strcpy(buffer, a->get_name().c_str());
	d_l_production->SetText(buffer);
	sprintf(buffer, "%i", a->get_production());
	d_l_duration->SetText(buffer);
	sprintf(buffer, "%i", a->get_max_strength());
	d_l_strength->SetText(buffer);
	sprintf(buffer, "%i", a->get_max_moves());
	d_l_moves->SetText(buffer);
	sprintf(buffer, "%i", a->get_upkeep());
	d_l_upkeep->SetText(buffer);
	sprintf(buffer, "%i", a->get_production_cost());
	d_l_production_cost->SetText(buffer);

	return true;
}
