//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*

#include "d_client.h"
#include "../common/client.h"

//#define DEBUG_IT

#ifdef DEBUG_IT
	#define debug(x) {x;}
#else
	#define debug(x)
#endif

#define cdbg cerr<<__FILE__<<": "<<__LINE__<<": "

D_Client::D_Client()
	:QDialog(0, "Client Dialog", true, 0)
{
	setCaption("FreeLords server selection");

	grid = new QGrid(3, QGrid::Vertical, this);
	grid->setSpacing(10);
	grid->setMargin(10);

	new QLabel("Host:", grid);
	new QLabel("Port:", grid);
	new QLabel("Status:", grid);

	e_host = new QLineEdit(grid);
	e_host->setText("localhost");
	e_port = new QLineEdit(grid);
	e_port->setFocus();
	l_status = new QLabel("not connected", grid);
	b_connect = new QPushButton("&Connect", grid);
	b_connect->setDefault(true);
	b_close = new QPushButton("C&lose", grid);

	connect(b_connect, SIGNAL(clicked()), this, SLOT(b_connect_clicked()));
	connect(b_close, SIGNAL(clicked()), this, SLOT(reject()));
	
	grid->adjustSize();
}

D_Client::~D_Client()
{
}

void D_Client::b_connect_clicked()
{
	b_connect->setEnabled(false);
	l_status->setText("trying to connect");
	
	client = new Client(e_host->text(), e_port->text().toInt());
	connect(client, SIGNAL(connected()), this, SLOT(client_connected()));
	connect(client, SIGNAL(error(int)), this, SLOT(client_error(int)));
}

void D_Client::client_connected()
{
	l_status->setText("connected to host");
	accept();
}

void D_Client::client_error(int error)
{
	QString temp;
	temp.setNum(error);
	temp.prepend("Error ");
	temp += " while connecting to host!";
	
	l_status->setText(temp);

	b_connect->setEnabled(true);
}
