//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "player_preferences.h"
#include <list>
#include "file.h"

// Constructor for the player preferences

Player_preferences::Player_preferences(Type type, std::string label, char* name,
		PG_Widget* parent, PG_Rect rect)
	:PG_ThemeWidget(parent, rect, true)
{   
	d_l_player = new PG_Label(this, PG_Rect(10, 1, 70, 20), label.c_str());
	PG_Rect r(5, 20, rect.w, rect.h - 10);	// this rect is the client-rect of the groupbox
	d_cb_active = new PG_CheckButton(this, 0, PG_Rect(r.x, r.y + 5, 70, 25), "Active");
	d_cb_computer = new PG_CheckButton(this, 0, PG_Rect(r.x + 75, r.y + 5, 120, 25), "Computer");

	if (type == HUMAN)
	{
		d_cb_active->SetPressed();
		d_cb_active->EnableReceiver(false);
		d_cb_computer->Hide();
	}
	else if (type == HUMAN_OR_COMPUTER)
	{
		d_cb_active->SetPressed();
		d_cb_active->EnableReceiver(false);
	}

	d_l_name = new PG_Label(this, PG_Rect(r.x + 5, r.y + 30, 60, 25), "Name:");
	d_e_name = new PG_LineEdit(this, PG_Rect(r.x + 70, r.y + 30, r.my_width - 80, 25));
	d_e_name->SetText(name);
	d_l_armyset = new PG_Label(this, PG_Rect(r.x + 5, r.y + 55, 150, 25), "Armyset:");
    d_cb_armyset = new PG_DropDown(this, 0, PG_Rect(r.x + 70, r.y + 55, r.my_width - 80, 25));
	d_cb_armyset->SetText("default");

	// fill the armyset combobox with names
	std::list<std::string> armysets = file::scanArmysets();
	for(std::list<std::string>::iterator it = armysets.begin(); it != armysets.end(); ++it)
	{
		d_cb_armyset->AddItem((*it).c_str());
	}
}

Player_preferences::~Player_preferences()
{
	delete d_l_player;
	delete d_l_armyset;
	delete d_l_name;
	delete d_e_name;
	delete d_cb_computer;
	delete d_cb_active;
	delete d_cb_armyset;
}

void Player_preferences::eventDraw(SDL_Surface* surface, const PG_Rect& rect)
{
	PG_ThemeWidget::eventDraw(surface, rect);
	DrawBorder(rect, 1);
	DrawHLine(0, 20, rect.w - 1, 255, 255, 255);
}

// End of file
