//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <iostream>
#include <pgdraw.h>
#include "smallmap.h"
#include "mapdef.h"
#include "mapset_tile.h"
#include "stacklist.h"
#include "citylist.h"
#include "city.h"
#include "ruin.h"
#include "temple.h"
#include "playerlist.h"
#include "player.h"

//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

SmallMap::SmallMap(PG_Widget* parent, PG_Rect rect, int x_squares, int y_squares)
	:PG_Widget(parent, rect, true), d_pressed(false), r(50), g(50), b(50), add(true)
{
	d_viewrect = new PG_Rect(1, 1, x_squares, y_squares);

	SDL_PixelFormat* fmt = GetScreenSurface()->format;
	d_static_map = SDL_CreateRGBSurface(SDL_SWSURFACE, 100, 100,
			fmt->BitsPerPixel, fmt->Rmask, fmt->Gmask,
			fmt->Bmask, fmt->Amask);
	
	// Draw map
	SDL_Color c;
	for(int i = 0; i < 100; i++)
	{
		for(int j = 0; j < 100; j++)
		{
			c = (*mapset)[mapdef->map[i][j]->get_type()]->get_color();
			PG_Draw::SetPixel(i+1, j+1, c.r, c.g, c.b, d_static_map);
		}
	}
    AddTimer(5);
}

SmallMap::~SmallMap()
{
	delete d_viewrect;
	SDL_FreeSurface(d_static_map);
}

void SmallMap::eventDraw(SDL_Surface* surface, const PG_Rect& rect)
{
	debug("eventDraw()");
	
	if (!inTimer)
	{
    	SDL_BlitSurface(d_static_map, 0, surface, PG_Rect(rect.x, rect.y, rect.w, rect.h).SDLRect());
    	DrawBorder(rect, 1);
    
    	// Draw all cities
    	for (Citylist::iterator it = citylist->begin(); it != citylist->end(); it++)
    	{	
    		SDL_Color c = (*it).get_player()->get_color();
    		DrawRectWH((*it).get_pos().x+1, (*it).get_pos().y+1, 3, 3, c.r, c.g, c.b);
    	}
    
    	// Draw all ruins
    	for (Ruinlist::iterator it = ruinlist->begin(); it != ruinlist->end(); it++)
    	{
    		// draw a yellow rect
    		DrawRectWH((*it).get_pos().x+1, (*it).get_pos().y+1, 2, 2, 255, 255, 0);	
    	}
    
    	// Draw temples
    	for (Templelist::iterator it = templelist->begin(); it != templelist->end(); it++)
    	{
    		// draw a magenta rect
    		DrawRectWH((*it).get_pos().x+1, (*it).get_pos().y+1, 2, 2, 255, 0, 255);
    	}
    
    	// Draw stacks
    	for (Stacklist::iterator it = stacklist->begin(); it != stacklist->end(); it++)
    	{
    		if ((*it)->get_player() != playerlist->get_neutral())
    		{
    			SDL_Color c = (*it)->get_player()->get_color();
    			DrawRectWH((*it)->get_pos().x+1, (*it)->get_pos().y+1, 2, 2, c.r, c.g, c.b);
    		}
   		}
	}
	drawViewrect();
}

// do some animation of the viewrect
void SmallMap::drawViewrect()
{
	if (r == 255) add = false;
	if (r == 160) add = true;
 
    if (add) {++r; ++g; ++b;}
    else {--r; --g; --b;} 
	
	// Draw the rectangle that shows the viewed part of the map
	DrawRectWH(d_viewrect->my_xpos, d_viewrect->my_ypos,
               d_viewrect->my_width, d_viewrect->my_height, r, g, b);
}

Uint32 SmallMap::eventTimer(PG_TimerID id, Uint32 interval)
{
    inTimer = true;
    Redraw();
    PG_TimerObject::eventTimer(id, interval);
    inTimer = false;
	return interval;
}

// Pressed in the drawing area to set viewrect

bool SmallMap::eventMouseButtonDown(const SDL_MouseButtonEvent* event)
{
	if(event->button == SDL_BUTTON_LEFT)
	{
		d_pressed = true;
		setViewrect(event->x, event->y);
	}
	return true;
}

// Released in the drawing area to set viewrect

bool SmallMap::eventMouseButtonUp(const SDL_MouseButtonEvent* event)
{
	if(event->button == SDL_BUTTON_LEFT) d_pressed = false;
	return true;
}

// This eventhandler is called when moving the mouse with pressed button in
// the drawing area to move viewrect

bool SmallMap::eventMouseMotion(const SDL_MouseMotionEvent* event)
{
	if (d_pressed) setViewrect(event->x, event->y);
	return true;
}

PARAGUI_CALLBACK(SmallMap::changed_viewrect)
{
	debug("changed_viewrect()");
	SDL_Rect* rect = (SDL_Rect*)clientdata;
	d_viewrect->my_xpos = rect->x;
	d_viewrect->my_ypos = rect->y;
	Redraw();
	return true;
}

void SmallMap::stack_moved()
{
	debug("stack_moved()");
	Redraw();
}

void SmallMap::setViewrect(int x, int y)
{
	x -= 5 + my_xpos;
	y -= 5 + my_ypos;

	if (x < 0) x = 0;
	if (y < 0) y = 0;
	if (x > (100 - d_viewrect->my_width)) x = 100 - d_viewrect->my_width;
	if (y > (100 - d_viewrect->my_height)) y = 100 - d_viewrect->my_height;

	d_viewrect->my_xpos = x;
	d_viewrect->my_ypos = y;

	Redraw();
	emit(MSG_SMALLMAP_MOVED_VIEWRECT);
}

