//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "splash.h"
#include <iostream>
#include <SDL_image.h>
#include "GameScenario.h"
#include "MainWindow.h"
#include "d_map.h"
#include "startup_screen.h"
#include "defs.h"
#include "mapset_tile.h"
#include "armysetlist.h"
#include "citylist.h"
#include "stacklist.h"
#include "playerlist.h"
#include "Configuration.h"

//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

Splash::Splash(PG_Application* app, const PG_Rect& rect)
	:PG_ThemeWidget(0, rect), d_gameScenario(0), d_mainWindow(0), d_app(app)
{
	SDL_Surface* bg = IMG_Load("../pic/various/splash_screen.jpg");
	SetBackground(bg, 2);
	
	d_b_campaign = new PG_Button(this, 0, PG_Rect(rect.w - 210, rect.h - 180, 180, 30), "New Campaign");
	d_b_singleplayer_game = new PG_Button(this, 0, PG_Rect(rect.w - 210, rect.h - 145, 180, 30), "New Game");
	d_b_multiplayer_game = new PG_Button(this, 0, PG_Rect(rect.w - 210, rect.h - 110, 180, 30), "New Multiplayer Game");
	d_b_load_game = new PG_Button(this, 0, PG_Rect(rect.w - 210, rect.h - 75, 180, 30), "Load Game");
	d_b_quit = new PG_Button(this, 0, PG_Rect(rect.w - 210, rect.h - 40, 180, 30), "Quit");

	d_b_campaign->SetEventObject(MSG_BUTTONCLICK, this, (MSG_CALLBACK_OBJ)&Splash::b_campaign_clicked);
	d_b_singleplayer_game->SetEventObject(MSG_BUTTONCLICK, this, (MSG_CALLBACK_OBJ)&Splash::b_singleplayer_game_clicked);
	d_b_multiplayer_game->SetEventObject(MSG_BUTTONCLICK, this, (MSG_CALLBACK_OBJ)&Splash::b_multiplayer_game_clicked);
	d_b_load_game->SetEventObject(MSG_BUTTONCLICK, this, (MSG_CALLBACK_OBJ)&Splash::b_load_game_clicked);
	d_b_quit->SetEventObject(MSG_BUTTONCLICK, this, (MSG_CALLBACK_OBJ)&Splash::b_quit_clicked);
}

Splash::~Splash()
{
	delete d_b_campaign;
	delete d_b_singleplayer_game;
	delete d_b_multiplayer_game;
	delete d_b_load_game;
	delete d_b_quit;
}

PARAGUI_CALLBACK(Splash::b_quit_clicked)
{
	d_app->Quit();
	return true;
}

PARAGUI_CALLBACK(Splash::b_singleplayer_game_clicked)
{
    newSinglePlayerGame();
	return true;
}

void Splash::newSinglePlayerGame()
{
	// should be somewhere else (or redesigned :o)
	// initialise "global" variables 
	armysetlist = new Armysetlist();
	citylist = new Citylist();
	ruinlist = new Ruinlist();
	templelist = new Templelist();
	mapset = new std::vector<Mapset_tile*>();
	playerlist = new Playerlist();
	stacklist = new Stacklist();

	playerlist->SetEventObject(MSG_PLAYERLIST_GAME_WON, this,
			(MSG_CALLBACK_OBJ)&Splash::gameFinished);

    GameScenario::MapType mapType;
    if (!Configuration::testing)
    {
	    D_Map d_map(this, PG_Rect((Width() - 250)/2, (Height() - 180)/2, 250, 180));
	    d_map.Show();
	    d_map.RunModal();
	    d_map.Hide();

     	mapType = d_map.getMapType();
    }
    else mapType = GameScenario::PREVIOUS;

    if (!Configuration::testing)
    {
    	Startup_screen startup_screen(this, 
            PG_Rect((Width() - 500)/2, (Height() - 500)/2, 500, 500));
    	startup_screen.Show();
    	startup_screen.RunModal();
    	startup_screen.Hide();
    }
    // Do a test scenario with 2 computer player here
    else
    {
		SDL_Color color;
		color.r = color.g = 0; color.b = 255;
		Player player1("Michael", "default", color);
        player1.set_computer(true);
		playerlist->push_back(player1);
		
  		color.r = 255; color.g = color.b = 0;
		Player player2("Susi", "default", color);
        player2.set_computer(true);
		playerlist->push_back(player2);
    }	
    Hide();
	
	d_gameScenario = new GameScenario(GameScenario::SINGLE_PLAYER, mapType);
	
	d_mainWindow = new MainWindow(d_gameScenario, PG_Rect(0, 0, Width(), Height()));
	d_mainWindow->SetEventObject(MSG_MAINWINDOW_QUIT, this,
        (MSG_CALLBACK_OBJ)&Splash::gameFinished);
	d_mainWindow->Show();
	d_mainWindow->start_game();
}

PARAGUI_CALLBACK(Splash::b_multiplayer_game_clicked)
{
	/*P
	game_scenario = new Game_Scenario();
	game_scenario->set_game_type(MULTI_PLAYER);
	game_scenario->set_map_type(RANDOM);
	turninfo = new Turninfo();

	D_Client d_client;
	if (!d_client.exec())
	{
		delete game_scenario;
		return;
	}

	Startup_screen startup_screen(0, "Start", TRUE);	
	if (!startup_screen.exec())
	{
		delete game_scenario;
		return;
	}

	hide();
	game_scenario->init();

	connect (game_scenario, SIGNAL(game_ended()), this, SLOT(game_finished()));
	*/
	return true;
}

PARAGUI_CALLBACK(Splash::b_load_game_clicked)
{
	std::cerr << "not implemented yet\n";
	return true;
}

PARAGUI_CALLBACK(Splash::b_campaign_clicked)
{
	std::cerr << "not implemented yet\n";
	return true;
}

PARAGUI_CALLBACK(Splash::gameFinished)
{
	debug("gameFinished()");
	
    delete d_gameScenario;
	delete d_mainWindow;
	delete citylist;
    delete ruinlist;
    delete templelist;
    delete playerlist;
    // still some delete's missing
    Show();
	return true;
}

