//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "w_edit.h"
#include <SDL_image.h>
#include <pgmessagebox.h>
#include "RuinSearchDialog.h"
#include "CityOccupationDialog.h"
#include "FightDialog.h"
#include "bigmap.h"
#include "smallmap.h"
#include "defs.h"
#include "army.h"
#include "GameScenario.h"
#include "stacklist.h"
#include "citylist.h"
#include "ruin.h"
#include "temple.h"
#include "playerlist.h"
#include "stackinfo.h"
#include "hero_offer.h"
#include "path.h"
#include "Configuration.h"

#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

W_Edit::W_Edit(GameScenario* gameScenario, PG_Widget* parent, PG_Rect rect)
        :PG_Widget(parent, rect), d_gameScenario(gameScenario)
{
        // this defines how many squares the bigmap shows! this is important
        // for the smallmap, because the viewrect depends on it
        int x_squares = (rect.my_width - 139) / 50;
        int y_squares = (rect.my_height - 110) / 50;

        // load button pictures (active/nonactive) 
        d_pic_move_nonactive = IMG_Load("../pic/various/button_move_nonactive.png");
        d_pic_defend_nonactive = IMG_Load("../pic/various/button_guard_nonactive.png");
	    d_pic_defendAndNext_nonactive = IMG_Load("../pic/various/button_guardnext_nonactive.png");
	    d_pic_search_nonactive = IMG_Load("../pic/various/button_search_nonactive.png");
	    d_pic_next_nonactive = IMG_Load("../pic/various/button_next_nonactive.png");
		d_pic_next = IMG_Load("../pic/various/button_next.png");
		d_pic_defend = IMG_Load("../pic/various/button_guard.png");
		d_pic_move = IMG_Load("../pic/various/button_move.png");
		d_pic_defendAndNext = IMG_Load("../pic/various/button_guardnext.png");
		d_pic_search = IMG_Load("../pic/various/button_search.png");
        d_pic_nextTurn = IMG_Load("../pic/various/button_endturn.png");
        
        d_smallmap = new SmallMap(this,
			PG_Rect(rect.my_width - 109, 20, 102, 102),
			x_squares, y_squares);
        d_bigmap = new BigMap(this, 
			PG_Rect(20, 20, x_squares*50, y_squares*50));
	d_bigmap->setViewrect(d_smallmap->getViewrect());
        d_stackinfo = new Stackinfo(this, 
			PG_Rect(20, rect.my_height - 80, 360, 72));

        l_turns = new PG_Label(this,
			PG_Rect(rect.my_width - 109, 130, 100, 20), "");
        l_gold = new PG_Label(this,
			PG_Rect(rect.my_width - 109, 150, 100, 20), "");

        b_move = new PG_Button(this, 1,
			PG_Rect(rect.my_width - 109, 170, 40, 40));
        b_next = new PG_Button(this, 2,
			PG_Rect(rect.my_width - 69, 170, 40, 40));
        b_defend = new PG_Button(this, 3,
			PG_Rect(rect.my_width - 109, 210, 40, 40));
        b_defend_and_next = new PG_Button(this, 4,
            PG_Rect(rect.my_width - 69, 210, 40, 40));
        b_search = new PG_Button(this, 5,
			PG_Rect(rect.my_width - 109, 250, 40, 40));
        b_next_turn = new PG_Button(this, 6,
			PG_Rect(rect.my_width - 69, 250, 40, 40));
	    b_next_turn->SetIcon(d_pic_nextTurn, 0, 0);

        // set callback functions here
        stacklist->SetEventObject(MSG_STACKLIST_STACK_MOVED, this,
			(MSG_CALLBACK_OBJ)&W_Edit::stacklist_stack_moved);
        stacklist->SetEventObject(MSG_STACKLIST_STACK_DIED, this,
			(MSG_CALLBACK_OBJ)&W_Edit::stacklist_stack_died);
	stacklist->SetEventObject(MSG_STACKLIST_CITY_OCCUPIED, this,
			(MSG_CALLBACK_OBJ)&W_Edit::city_occupied);
	stacklist->SetEventObject(MSG_STACKLIST_REDRAW, this,
			(MSG_CALLBACK_OBJ)&W_Edit::stacklist_stack_redraw);

	// this has to be done for each player!!!
	for (Playerlist::iterator it = playerlist->begin();
			it != playerlist->end(); ++it)
	{
		(*it).SetEventObject(MSG_PLAYER_STATUS_CHANGED, this,
				     (MSG_CALLBACK_OBJ)&W_Edit::update_status);
		(*it).SetEventObject(MSG_PLAYER_LOCK_SCREEN, this,
				(MSG_CALLBACK_OBJ)&W_Edit::lock_screen);
		(*it).SetEventObject(MSG_PLAYER_UNLOCK_SCREEN, this,
				(MSG_CALLBACK_OBJ)&W_Edit::unlock_screen);
	}

        d_smallmap->SetEventObject(MSG_SMALLMAP_MOVED_VIEWRECT, this,
			(MSG_CALLBACK_OBJ)&W_Edit::bigmap_redraw);
	
        d_bigmap->SetEventObject(MSG_BIGMAP_MOVED_VIEWRECT, this,
			(MSG_CALLBACK_OBJ)&W_Edit::smallmap_redraw);
	d_bigmap->SetEventObject(MSG_BIGMAP_STACK_SELECTED, this,
			(MSG_CALLBACK_OBJ)&W_Edit::bigmap_stack_selected);

        b_move->SetEventObject(MSG_BUTTONCLICK, this,
			(MSG_CALLBACK_OBJ)&W_Edit::b_move_clicked);
        b_next->SetEventObject(MSG_BUTTONCLICK, this,
			(MSG_CALLBACK_OBJ)&W_Edit::b_next_clicked);
        b_defend->SetEventObject(MSG_BUTTONCLICK, this,
			(MSG_CALLBACK_OBJ)&W_Edit::b_defend_clicked);
        b_defend_and_next->SetEventObject(MSG_BUTTONCLICK, this,
			(MSG_CALLBACK_OBJ)&W_Edit::b_defend_and_next_clicked);
        b_search->SetEventObject(MSG_BUTTONCLICK, this,
			(MSG_CALLBACK_OBJ)&W_Edit::b_search_clicked);
        b_next_turn->SetEventObject(MSG_BUTTONCLICK, this,
			(MSG_CALLBACK_OBJ)&W_Edit::b_next_turn_clicked);

	//some other callbacks
	playerlist->SetEventObject(MSG_PLAYERLIST_NEXT_TURN, this,
			(MSG_CALLBACK_OBJ)&W_Edit::next_turn);
			
    all_defending = false;
    check_buttons();
}

W_Edit::~W_Edit()
{
	delete d_smallmap;
	delete d_bigmap;
	delete d_stackinfo;
	delete l_turns;
	delete l_gold;
	SDL_FreeSurface(d_pic_move_nonactive);
	SDL_FreeSurface(d_pic_defend_nonactive);
	SDL_FreeSurface(d_pic_defendAndNext_nonactive);
	SDL_FreeSurface(d_pic_search_nonactive);
	SDL_FreeSurface(d_pic_next_nonactive);
	SDL_FreeSurface(d_pic_next);
	SDL_FreeSurface(d_pic_defend);
	SDL_FreeSurface(d_pic_move);
	SDL_FreeSurface(d_pic_defendAndNext);
	SDL_FreeSurface(d_pic_search);
    SDL_FreeSurface(d_pic_nextTurn);
    delete b_next;
	delete b_move;
	delete b_defend;
	delete b_defend_and_next;
	delete b_search;
	delete b_next_turn;
}

PARAGUI_CALLBACK(W_Edit::b_next_turn_clicked)
{
	playerlist->get_activeplayer()->next_turn();
	return true;
}

PARAGUI_CALLBACK(W_Edit::next_turn)
{
	debug("next_turn()");

	if (!playerlist->get_activeplayer())
		return true;

	if (playerlist->get_activeplayer() == &(*playerlist->begin()))
	{
		d_gameScenario->nextTurn();
	}

	d_bigmap->stack_deselected();
	d_stackinfo->Hide();
	all_defending = false;
	display_first_city();
	update_status();
	
	// human player
	if (!playerlist->get_activeplayer()->isComputer())
    {
        if (Configuration::showNextPlayer) picture_next_player();
	}

	//quick hack: offer every player a hero on occasion, replace it
	//with something more generic ASAP
	if (rand() % 15 == 7)
	{
		hero_joins(false);
		d_bigmap->Redraw();
	}

	// computer player
	if (playerlist->get_activeplayer()->isComputer())
	{
        playerlist->get_activeplayer()->computerTurn();
        playerlist->get_activeplayer()->next_turn();
    }
    
	return true;
}

// Print new status
void W_Edit::update_status()
{
	char buffer[80];
	sprintf(buffer, "Turns: %i", d_gameScenario->getTurns());
	l_turns->SetText(buffer);

	sprintf(buffer, "Gold: %i", playerlist->get_activeplayer()->getGold());
	l_gold->SetText(buffer);
}

// find first city of activeplayer and view it if outside screen
void W_Edit::display_first_city()
{
	debug("display_first_city()");

	for (Citylist::iterator it = citylist->begin();
			it != citylist->end(); it++)
	{
		if ((*it).get_player() == playerlist->get_activeplayer())
		{
			d_bigmap->center_view((*it).get_pos());
			d_bigmap->Redraw();
			break;
		}
	}
}

// pop up message window for next player
void W_Edit::picture_next_player()
{
	debug("picture_next_player()");

	char buffer[50];
	sprintf(buffer, "Next Player: %s",
			playerlist->get_activeplayer()->get_name().c_str());

	int x = (my_width - 200) / 2;
	int y = (my_height - 110) / 2;

	PG_MessageBox mb(0, PG_Rect(x, y, 200, 110), "Next Player", buffer,
			PG_Rect(60, 70, 80, 30), "OK");
	mb.Show();
	mb.WaitForClick();	// when button clicked -> continue
	mb.Hide();
}

void W_Edit::hero_joins(bool free)
{
	debug("hero_joins()");

	// Calculate how much the hero will cost (partly random based)
	int needed_gold = 0;
	if (!free) needed_gold = 800 + rand() % 700;

	// check if the player has enough money enough gold
	if (playerlist->get_activeplayer()->getGold() >= needed_gold)
	{
	    if (!d_lock)
	    {
		    Hero_offer hero_offer(needed_gold, this);
		    hero_offer.Show();
		    hero_offer.RunModal();
		    hero_offer.Hide();
		}
		else
		{
		    City* city;
			// first city of activeplayer
        	for (Citylist::iterator it = citylist->begin();
        			it != citylist->end(); it++)
        	{
        		if ((*it).get_player() == playerlist->get_activeplayer())
        		{
        			city = &(*it);
        			break;
        		}
        	}
			city->add_hero(0, "ComputerLord");
			playerlist->get_activeplayer()->withdrawGold(needed_gold);
		}
		update_status();
	}
}

PARAGUI_CALLBACK(W_Edit::b_next_clicked)
{
	Stack* stack = stacklist->set_next();
	if (stack) d_bigmap->stack_selected();
	else all_defending = true;
	check_buttons();
	return true;
}

PARAGUI_CALLBACK(W_Edit::b_move_clicked)
{
	stacklist->move(stacklist->get_activestack());
	check_buttons();
	return true;
}

PARAGUI_CALLBACK(W_Edit::b_defend_clicked)
{
	stacklist->get_activestack()->set_defending(true);
	d_bigmap->stack_deselected();
	d_stackinfo->Hide();
	check_buttons();

	return true;
}

PARAGUI_CALLBACK(W_Edit::b_defend_and_next_clicked)
{
	b_defend_clicked(GetID(), this, 1, 0);
	b_next_clicked(GetID(), this, 1, 0);
	return true;
}

PARAGUI_CALLBACK(W_Edit::b_search_clicked)
{
	Stack* stack = stacklist->get_activestack();
	assert(stack->has_hero());

	Player* p = playerlist->get_activeplayer();
	Ruin* ruin = ruinlist->get_object_at(stack->get_pos());
	Temple* temple = templelist->get_object_at(stack->get_pos());

	if (ruin)
	{
		int cur_gold = p->getGold();

		if (!(p->stack_search_ruin(stack, ruin)))
		{
			if (stack->empty())
			{
				stacklist_stack_died(GetID(), this, 1, 0);
				stacklist->fl_remove(stack);
				return true;
			}
			stacklist_stack_redraw(GetID(), this, 1, 0);
			return true;
		}

		stacklist_stack_redraw(GetID(), this, 1, 0);

		//this also includes the gold-only hack
		int gold_added = p->getGold() - cur_gold;

		RuinSearchDialog dialog(this, gold_added);
		dialog.Show();
		dialog.RunModal();
		dialog.Hide();

		update_status();
	}
	
	else if ((temple) && (temple->search()))
	{
		PG_MessageBox mb(this, PG_Rect(x, y, 200, 110), "Temple",
				"Your armies have been blessed",
				PG_Rect(60, 70, 80, 30), "OK");
		mb.Show();
		mb.WaitForClick();
		mb.Hide();

		p->stack_visit_temple(stack, temple);
	}

	return true;
}

PARAGUI_CALLBACK(W_Edit::stacklist_stack_moved)
{
	d_bigmap->stack_moved();
	d_smallmap->stack_moved();
	d_stackinfo->read_data();
	check_buttons();
    return true;
}

PARAGUI_CALLBACK(W_Edit::stacklist_stack_died)
{
    debug("stacklist_stack_died()");
	d_bigmap->stack_deselected();
	d_smallmap->Redraw();
	d_stackinfo->Hide();
	check_buttons();
	return true;
}

PARAGUI_CALLBACK(W_Edit::stacklist_stack_redraw)
{
	d_bigmap->Redraw();
	d_stackinfo->read_data();
	check_buttons();
	return true;
}

PARAGUI_CALLBACK(W_Edit::bigmap_stack_selected)
{
	d_stackinfo->Show();
	d_stackinfo->read_data();
	check_buttons();
    return true;
}

PARAGUI_CALLBACK(W_Edit::city_occupied)
{
	debug("city_occupied()");

	// Get the occupied city
	City* city = citylist->get_object_at(stacklist->get_activestack()->get_pos());

	// if a computer makes it's turn and occupied a city, we shouldn't
    // show a modal dialog :)

	if (!d_lock)
	{
	    CityOccupationDialog dialog(city);
	    dialog.Show();
	    dialog.RunModal();
	    dialog.Hide();
    }
    else
    {
    	playerlist->get_activeplayer()->city_occupy(city);
    }
    
	playerlist->check_players();
	d_bigmap->Redraw();
	d_stackinfo->read_data();
	check_buttons();

	return true;
}

PARAGUI_CALLBACK(W_Edit::bigmap_redraw)
{
	d_bigmap->Redraw();
	return true;
}

PARAGUI_CALLBACK(W_Edit::smallmap_redraw)
{
	d_smallmap->Redraw();
	return true;
}

PARAGUI_CALLBACK(W_Edit::lock_screen)
{
	debug("locking screen...")

	//don't accept modifying user input from now on
	d_bigmap->set_enable(false);

	b_next_turn->EnableReceiver(false);
	b_next->EnableReceiver(false);
	b_defend->EnableReceiver(false);
	b_move->EnableReceiver(false);
	b_search->EnableReceiver(false);
	b_defend_and_next->EnableReceiver(false);

	d_lock = true;
	
	return true;
}

PARAGUI_CALLBACK(W_Edit::unlock_screen)
{
	debug("unlocking screen...")

	//the inverse of lock_screen()
	b_next_turn->EnableReceiver(true);
	d_bigmap->set_enable(true);
	d_lock = false;
	check_buttons();

	return true;
}

void W_Edit::check_buttons()
{
    if (d_lock) return;
    
	debug("check_buttons()");

	// first assume that all buttons are disabled
	b_move->EnableReceiver(false);
	b_move->SetIcon(d_pic_move_nonactive, 0, 0);
	b_defend->EnableReceiver(false);
	b_defend->SetIcon(d_pic_defend_nonactive, 0, 0);
	b_defend_and_next->EnableReceiver(false);
	b_defend_and_next->SetIcon(d_pic_defendAndNext_nonactive, 0, 0);
	b_search->EnableReceiver(false);
	b_search->SetIcon(d_pic_search_nonactive, 0, 0);
	b_next->EnableReceiver(false);
	b_next->SetIcon(d_pic_next_nonactive, 0, 0);

	// now check which buttons should be enabled
	if (!all_defending) 
	{
		b_next->EnableReceiver(true);
		b_next->SetIcon(d_pic_next, 0, 0);
	}

	Stack* stack = stacklist->get_activestack();
	if (stack)
	{
		b_defend->EnableReceiver(true);
		b_defend->SetIcon(d_pic_defend, 0, 0);

		if (stack->get_path()->size() > 0 && stack->enough_moves_next_step())
		{
			b_move->EnableReceiver(true);
			b_move->SetIcon(d_pic_move, 0, 0);
		}
		//TBD what if there is only 1 army and it isn't defending?
		// should be something like (!all defend) && (further_stacks)
		if (!all_defending)
		{
			b_defend_and_next->EnableReceiver(true);
			b_defend_and_next->SetIcon(d_pic_defendAndNext, 0, 0);
		}

		if (stack->has_hero())
		{
			Ruin* ruin = ruinlist->get_object_at(stack->get_pos());
			if ((ruin && !ruin->isSearched()) || templelist->get_object_at(stack->get_pos()))
			{
				b_search->EnableReceiver(true);
				b_search->SetIcon(d_pic_search, 0, 0);
			}
		}
	}
	Redraw();
}

void W_Edit::start_game()
{
	debug ("start_game()");
	playerlist->next_player();
}

