//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <paragui.h>
#include <pgapplication.h>
#include <iostream>
#include <stdlib.h>
#include <time.h>
#include "splash.h"
#include "Configuration.h"

SDL_Rect* get_best_resolution(Uint32 mymode)
{
	SDL_Rect **modes;
	int i;

	// Get available fullscreen/hardware modes
	modes = SDL_ListModes(NULL, mymode);

	// Check if there are any modes available
	if(modes == (SDL_Rect **)0)
	{
		cerr << "No modes available!\n";
		exit(-1);
	}

	// Check if any resolution is ok
	if(modes == (SDL_Rect **)-1)
	{
		cout << "All resolutions available.\n";
		return 0;
	}
	else
	{
		// Print valid modes
		cout << "\nAvailable Modes\n";

		for(i = 0; modes[i]; ++i)
		{
			cout << "Resolution = " << modes[i]->w << "x" << modes[i]->h << endl;
		}
		if ((i == 0) && (modes[i]->w == 640))
		{
			cout << "You need at least 800 x 600 resolution to run FreeLords!\n";
			exit(-1);
		}
	}

	return modes[0];	// return the highest resolution
}


int main(int argc, char* argv[])
{
	char version[10];
    strcpy(version, "0.1.6");
    
    char theme[20];
    strcpy(theme, "default");
    	
	srand(time(NULL));		 // set the random seed

	int WIDTH = 800;
    int HEIGHT = 600;
    bool resolutionFixed = false;
	Uint32 flags = SDL_SWSURFACE;

	if (argc > 1)
	{
		for (int i = argc; i > 1; i--)
		{
			string parameter(argv[i-1]); 
			if (parameter == "--fullscreen" || parameter == "-f")
			{
				flags |= SDL_FULLSCREEN;
			}
			if (parameter == "-r1280")
			{
				WIDTH = 1280;
				HEIGHT = 1024;
				resolutionFixed = true;
			}
			if (parameter == "-r1024")
			{
				WIDTH = 1024;
				HEIGHT = 786;
				resolutionFixed = true;
			}
			if (parameter == "-r800")
			{
				WIDTH = 800;
				HEIGHT = 600;
				resolutionFixed = true;
			}
			if (parameter == "-t" || parameter == "--test")
			{
                Configuration::testing = true;
			}
			if (parameter == "--help" || parameter == "-h")
			{
				cout << endl;
				cout << "FreeLords " << version << endl << endl;
				cout << "Available parameters:" << endl << endl; 
				cout << "-h,      --help		     Shows this help screen\n";
				cout << "-r1280                      Resolution set to 1280x1024" << endl;
				cout << "-r1024                      Resolution set to 1024x786" << endl;
				cout << "-r800                       Resolution set to 800x600" << endl;
                cout << "-f,      --fullscreen	     Fullscreen mode" << endl;
				cout << "-t,      --test	         Starting with a test-scenario" << endl << endl << endl;
				cout << "Please send bug reports to midramas@freelords.org" << endl << endl;
				exit(0);
			}
		}
	}

	PG_Application app;
	app.SetEmergencyQuit(true);
	
	if (!app.LoadTheme("default", true, "../dat"))
	{
        cerr << "Unable to load theme: " << theme << endl;
        exit(-1);
	}
	
    app.SetFontSize(14);

    if (!resolutionFixed)
    {
        SDL_Rect* mode = get_best_resolution(flags);
        if (mode != 0)
        {
       		WIDTH = mode->w;
	       	HEIGHT = mode->h;
       	}
   	}   	

    cout << "Using resoluting: " << WIDTH << "x" << HEIGHT << endl;	
	
	if (!app.InitScreen(WIDTH, HEIGHT, 16, flags))
	{
		cerr << "Couldn't initialize X Windowing System\n";
		exit(1);
	}

	char title[50];
	sprintf(title, "FreeLords %s", version);
		
	SDL_WM_SetCaption(title, "");

	// Initialize the SDL Timer sub-system
	SDL_InitSubSystem(SDL_INIT_TIMER);

	Splash splash(&app, PG_Rect(0, 0, WIDTH, HEIGHT));
	splash.Show();
	
	if (Configuration::testing)
	{
        splash.newSinglePlayerGame();
    }

	app.Run();

	return EXIT_SUCCESS;
}
